@@ -29,6 +29,13 @@ import traceback
 
 import re
 
+if sys.platform  == "darwin":
+    try:
+        import certifi
+        os.environ["SSL_CERT_FILE"] = certifi.where()
+    except ImportError:
+        pass
+
 if sys.platform == 'win32':
     try:
         import winreg
@@ -40,17 +47,18 @@ if sys.platform == 'win32':
         #  #wxversion.ensureMinimal('2.8-unicode', optionsRequired=True)
         #  #wxversion.select('2.8-unicode', optionsRequired=True)
         #  #  wxversion.ensureMinimal('4.0')
-        
+
+# Forcing to use X11, OpenGL in wxPython doesn't work with Wayland.
+if sys.platform not in ("win32", "darwin"):
+    os.environ["GDK_BACKEND"] = "x11"
+
 import wx
 try:
     from wx.adv import SplashScreen
 except ImportError:
     from wx import SplashScreen
-#from wx.lib.pubsub import setupv1 #new wx
-#  from wx.lib.pubsub import setuparg1# as psv1
-#from wx.lib.pubsub import Publisher 
-#import wx.lib.pubsub as ps
-from wx.lib.pubsub import pub as Publisher
+
+from invesalius.pubsub import pub as Publisher
 
 #import wx.lib.agw.advancedsplash as agw
 #if sys.platform.startswith('linux'):
@@ -68,6 +76,7 @@ import invesalius.utils as utils
 from invesalius import inv_paths
 
 FS_ENCODE = sys.getfilesystemencoding()
+LANG = None
 
 # ------------------------------------------------------------------
 
@@ -81,6 +90,17 @@ if sys.platform in ('linux2', 'linux', 'win32'):
         del tmp_var
 
 
+session = ses.Session()
+if session.ReadSession():
+    lang = session.GetLanguage()
+    if lang:
+        try:
+            _ = i18n.InstallLanguage(lang)
+            LANG = lang
+        except FileNotFoundError:
+            pass
+
+
 class InVesalius(wx.App):
     """
     InVesalius wxPython application class.
@@ -89,7 +109,7 @@ class InVesalius(wx.App):
         """
         Initialize splash screen and main frame.
         """
-        
+
         from multiprocessing import freeze_support
         freeze_support()
 
@@ -121,39 +141,29 @@ class Inv3SplashScreen(SplashScreen):
     Splash screen to be shown in InVesalius initialization.
     """
     def __init__(self):
-        # Splash screen image will depend on currently language
-        lang = False
-
-        self.locale = wx.Locale(wx.LANGUAGE_DEFAULT)
+        # Splash screen image will depend on the current language
+        lang = LANG
+        self.locale = wx.Locale(wx.LANGUAGE_ENGLISH)
 
         # Language information is available in session configuration
         # file. First we need to check if this file exist, if now, it
         # should be created
-        create_session = False
-        session = ses.Session()
-        if not (session.ReadSession()):
-            create_session = True
+        create_session = LANG is None
 
-        install_lang = 0
-        lang = session.GetLanguage()
+        install_lang = False
         if lang:
-            if (lang != "False"):
-                _ = i18n.InstallLanguage(lang)
-                install_lang = 1
-            else:
-                install_lang = 0
-        else:
-            install_lang = 0
+            _ = i18n.InstallLanguage(lang)
+            install_lang = True
 
         # If no language is set into session file, show dialog so
         # user can select language
-        if install_lang == 0:
+        if not install_lang:
             dialog = lang_dlg.LanguageDialog()
 
             # FIXME: This works ok in linux2, darwin and win32,
             # except on win64, due to wxWidgets bug
             try:
-                ok = (dialog.ShowModal() == wx.ID_OK)
+                ok = dialog.ShowModal() == wx.ID_OK
             except wx._core.PyAssertionError:
                 ok = True
             finally:
@@ -169,27 +179,25 @@ class Inv3SplashScreen(SplashScreen):
 
             dialog.Destroy()
 
-        # Session file should be created... So we set the recent
-        # choosen language
-        if (create_session):
+        # Session file should be created... So we set the recently chosen language.
+        if create_session:
             session.CreateItens()
             session.SetLanguage(lang)
             session.WriteSessionFile()
 
         #  session.SaveConfigFileBackup()
 
-           
-        # Only after language was defined, splash screen will be
-        # shown
+
+        # Only after language was defined, splash screen will be shown.
         if lang:
-            
+
             #import locale
             #try:
             #    locale.setlocale(locale.LC_ALL, '')
             #except locale.Error:
             #    pass
-            
-            
+
+
             # For pt_BR, splash_pt.png should be used
             if (lang.startswith('pt')):
                 icon_file = "splash_pt.png"
@@ -201,13 +209,11 @@ class Inv3SplashScreen(SplashScreen):
                 abs_file_path = os.path.abspath(".." + os.sep)
                 path = abs_file_path
                 path = os.path.join(path, 'icons', icon_file)
-            
             else:
-
-                path = os.path.join(".","icons", icon_file)
+                path = os.path.join(inv_paths.ICON_DIR, icon_file)
                 if not os.path.exists(path):
-                    path = os.path.join(".", "icons", "splash_en.png")
-				
+                    path = os.path.join(inv_paths.ICON_DIR, "splash_en.png")
+
             bmp = wx.Image(path).ConvertToBitmap()
 
             try:
@@ -222,7 +228,7 @@ class Inv3SplashScreen(SplashScreen):
                                   id=-1,
                                   parent=None)
             self.Bind(wx.EVT_CLOSE, self.OnClose)
-            wx.Yield()
+            wx.GetApp().Yield()
             wx.CallLater(200, self.Startup)
 
     def Startup(self):
@@ -231,12 +237,12 @@ class Inv3SplashScreen(SplashScreen):
         from invesalius.gui.frame import Frame
         from invesalius.control import Controller
         from invesalius.project import Project
-        
+
         self.main = Frame(None)
         self.control = Controller(self.main)
-        
+
         self.fc = wx.CallLater(200, self.ShowMain)
-        options, args = parse_comand_line()
+        options, args = parse_command_line()
         wx.CallLater(1, use_cmd_optargs, options, args)
 
         # Check for updates
@@ -265,7 +271,10 @@ class Inv3SplashScreen(SplashScreen):
 
 
 def non_gui_startup(options, args):
-    lang = 'en'
+    if LANG:
+        lang = LANG
+    else:
+        lang = 'en'
     _ = i18n.InstallLanguage(lang)
 
     from invesalius.control import Controller
@@ -284,7 +293,7 @@ def non_gui_startup(options, args):
 # ------------------------------------------------------------------
 
 
-def parse_comand_line():
+def parse_command_line():
     """
     Handle command line arguments.
     """
@@ -294,7 +303,7 @@ def parse_comand_line():
     # Parse command line arguments
     parser = op.OptionParser()
 
-    # -d or --debug: print all pubsub messagessent
+    # -d or --debug: print all pubsub messages sent
     parser.add_option("-d", "--debug",
                       action="store_true",
                       dest="debug")
@@ -312,6 +321,12 @@ def parse_comand_line():
     parser.add_option("--import-all",
                       action="store")
 
+    parser.add_option("--import-folder", action="store", dest="import_folder")
+
+    parser.add_option("--remote-host",
+                      action="store",
+                      dest="remote_host")
+
     parser.add_option("-s", "--save",
                       help="Save the project after an import.")
 
@@ -331,17 +346,14 @@ def parse_comand_line():
                       dest="save_masks", default=True,
                       help="Make InVesalius not export mask when exporting project.")
 
+    parser.add_option("--use-pedal", action="store_true", dest="use_pedal",
+                      help="Use an external trigger pedal")
+
     options, args = parser.parse_args()
     return options, args
 
 
 def use_cmd_optargs(options, args):
-    # If debug argument...
-    if options.debug:
-        Publisher.subscribe(print_events, Publisher.ALL_TOPICS)
-        session = ses.Session()
-        session.debug = 1
-
     # If import DICOM argument...
     if options.dicom_dir:
         import_dir = options.dicom_dir
@@ -354,6 +366,13 @@ def use_cmd_optargs(options, args):
         check_for_export(options)
 
         return True
+    elif options.import_folder:
+        Publisher.sendMessage('Import folder', folder=options.import_folder)
+        if options.save:
+            Publisher.sendMessage('Save project', filepath=os.path.abspath(options.save))
+            exit(0)
+        check_for_export(options)
+
     elif options.import_all:
         import invesalius.reader.dicom_reader as dcm
         for patient in dcm.GetDicomGroups(options.import_all):
@@ -415,7 +434,7 @@ def check_for_export(options, suffix='', remove_surfaces=False):
         try:
             from invesalius.project import Project
 
-            for threshold_name, threshold_range in Project().presets.thresh_ct.iteritems():
+            for threshold_name, threshold_range in Project().presets.thresh_ct.items():
                 if isinstance(threshold_range[0], int):
                     path_ = u'{}-{}-{}.stl'.format(options.export_to_all, suffix, threshold_name)
                     export(path_, threshold_range, remove_surface=True)
@@ -470,22 +489,16 @@ def print_events(topic=Publisher.AUTO_TOPIC, **msg_data):
     """
     utils.debug("%s\n\tParameters: %s" % (topic, msg_data))
 
-def main():
-    """
-    Initialize InVesalius GUI
-    """
-    options, args = parse_comand_line()
 
-    if options.no_gui:
-        non_gui_startup(options, args)
-    else:
-        application = InVesalius(0)
-        application.MainLoop()
+def init():
+    """
+    Initialize InVesalius.
 
-if __name__ == '__main__':
+    Mostly file-system related initializations.
+    """
     #Is needed because of pyinstaller
     multiprocessing.freeze_support()
-    
+
     #Needed in win 32 exe
     if hasattr(sys,"frozen") and sys.platform.startswith('win'):
 
@@ -508,12 +521,53 @@ if __name__ == '__main__':
         path = inv_paths.USER_LOG_DIR.join("stderr.log")
         sys.stderr = open(path, "w")
 
-    # Add current directory to PYTHONPATH, so other classes can
-    # import modules as they were on root invesalius folder
-    sys.path.insert(0, '.')
-    sys.path.append(".")
+
+def main(connection=None):
+    """
+    Start InVesalius.
+
+    Parameters:
+        connection: An object to communicate with the outside world.
+          In theory, can be any object supports certain function calls.
+          See invesalius.net.neuronavigation_api for a comprehensive
+          description of how the object is used.
+
+          Note that if InVesalius is started in the usual way by running
+          app.py, the connection object defaults to None. To enable this
+          functionality, InVesalius needs to be started by calling the main
+          function directly with a proper connection object.
+    """
+    init()
+
+    options, args = parse_command_line()
+
+    session = ses.Session()
+    session.debug = options.debug
+
+    if options.debug:
+        Publisher.subscribe(print_events, Publisher.ALL_TOPICS)
+
+    if options.remote_host is not None:
+        from invesalius.net.remote_control import RemoteControl
+
+        remote_control = RemoteControl(options.remote_host)
+        remote_control.connect()
+
+    if options.use_pedal:
+        from invesalius.net.pedal_connection import PedalConnection
+
+        PedalConnection().start()
+
+    from invesalius.net.neuronavigation_api import NeuronavigationApi
+
+    NeuronavigationApi(connection)
+
+    if options.no_gui:
+        non_gui_startup(options, args)
+    else:
+        application = InVesalius(0)
+        application.MainLoop()
 
 
-    # Init application
+if __name__ == '__main__':
     main()
-
