@@ -1,6 +1,6 @@
 # Singleton and Publisher-Subscriber design patterns example.
 
-import wx.lib.pubsub as ps
+from invesalius.pubsub import pub as Publisher
 
 class Singleton(type):
     # This is a Gary Robinson implementation:
