@@ -19,6 +19,7 @@
 
 import os.path
 import platform
+import psutil
 import sys
 import wx
 import itertools
@@ -27,7 +28,7 @@ from invesalius import utils
 from invesalius import inv_paths
 
 #from invesalius.project import Project
-INVESALIUS_VERSION = "3.1.99994"
+INVESALIUS_VERSION = "3.1.99998"
 
 INVESALIUS_ACTUAL_FORMAT_VERSION = 1.1
 
@@ -467,11 +468,11 @@ VTK_WARNING = 0
 #----------------------------------------------------------
 
 [ID_DICOM_IMPORT, ID_PROJECT_OPEN, ID_PROJECT_SAVE_AS, ID_PROJECT_SAVE,
- ID_PROJECT_CLOSE, ID_EXPORT_SLICE, ID_EXPORT_MASK, ID_PROJECT_INFO,
+ ID_PROJECT_CLOSE, ID_EXPORT_SLICE, ID_PROJECT_PROPERTIES, ID_EXPORT_MASK, ID_PROJECT_INFO,
  ID_SAVE_SCREENSHOT, ID_DICOM_LOAD_NET, ID_PRINT_SCREENSHOT,
  ID_IMPORT_OTHERS_FILES, ID_PREFERENCES, ID_DICOM_NETWORK, ID_TIFF_JPG_PNG,
  ID_VIEW_INTERPOLATED, ID_MODE_NAVIGATION, ID_ANALYZE_IMPORT, ID_NIFTI_IMPORT,
- ID_PARREC_IMPORT, ID_MODE_DBS] = [wx.NewId() for number in range(21)]
+ ID_PARREC_IMPORT, ID_MODE_DBS] = [wx.NewId() for number in range(22)]
 ID_EXIT = wx.ID_EXIT
 ID_ABOUT = wx.ID_ABOUT
 
@@ -486,6 +487,7 @@ ID_ABOUT = wx.ID_ABOUT
     [wx.NewId() for number in range(3)]
 
 ID_START = wx.NewId()
+ID_PLUGINS_SHOW_PATH = wx.NewId()
 
 ID_FLIP_X = wx.NewId()
 ID_FLIP_Y = wx.NewId()
@@ -507,15 +509,28 @@ ID_MANUAL_SEGMENTATION = wx.NewId()
 ID_WATERSHED_SEGMENTATION = wx.NewId()
 ID_THRESHOLD_SEGMENTATION = wx.NewId()
 ID_FLOODFILL_SEGMENTATION = wx.NewId()
+ID_FLOODFILL_SEGMENTATION = wx.NewId()
+ID_SEGMENTATION_BRAIN = wx.NewId()
 ID_CROP_MASK = wx.NewId()
 ID_DENSITY_MEASURE = wx.NewId()
 ID_MASK_DENSITY_MEASURE = wx.NewId()
 ID_CREATE_SURFACE = wx.NewId()
 ID_CREATE_MASK = wx.NewId()
+ID_MASK_3D_PREVIEW = wx.NewId()
+ID_MASK_3D_RELOAD = wx.NewId()
+ID_MASK_3D_AUTO_RELOAD = wx.NewId()
 
 ID_GOTO_SLICE = wx.NewId()
 ID_GOTO_COORD = wx.NewId()
 
+ID_MANUAL_WWWL = wx.NewId()
+
+# Tractography with Trekker
+ID_TREKKER_MASK = wx.NewId()
+ID_TREKKER_IMG = wx.NewId()
+ID_TREKKER_FOD = wx.NewId()
+ID_TREKKER_ACT = wx.NewId()
+
 #---------------------------------------------------------
 STATE_DEFAULT = 1000
 STATE_WL = 1001
@@ -540,6 +555,7 @@ SLICE_STATE_REMOVE_MASK_PARTS = 3012
 SLICE_STATE_SELECT_MASK_PARTS = 3013
 SLICE_STATE_FFILL_SEGMENTATION = 3014
 SLICE_STATE_CROP_MASK = 3015
+SLICE_STATE_TRACTS = 3016
 
 VOLUME_STATE_SEED = 2001
 #  STATE_LINEAR_MEASURE = 3001
@@ -554,7 +570,7 @@ TOOL_STATES = [STATE_WL, STATE_SPIN, STATE_ZOOM,
 
 
 TOOL_SLICE_STATES = [SLICE_STATE_CROSS, SLICE_STATE_SCROLL,
-                     SLICE_STATE_REORIENT]
+                     SLICE_STATE_REORIENT, SLICE_STATE_TRACTS]
 
 
 SLICE_STYLES = TOOL_STATES + TOOL_SLICE_STATES
@@ -570,11 +586,6 @@ SLICE_STYLES.append(STATE_MEASURE_DENSITY)
 SLICE_STYLES.append(STATE_MEASURE_DENSITY_ELLIPSE)
 SLICE_STYLES.append(STATE_MEASURE_DENSITY_POLYGON)
 
-VOLUME_STYLES = TOOL_STATES + [VOLUME_STATE_SEED, STATE_MEASURE_DISTANCE,
-        STATE_MEASURE_ANGLE]
-VOLUME_STYLES.append(STATE_DEFAULT)
-
-
 STYLE_LEVEL = {SLICE_STATE_EDITOR: 1,
                SLICE_STATE_WATERSHED: 1,
                SLICE_STATE_MASK_FFILL: 2,
@@ -644,8 +655,26 @@ BOOLEAN_DIFF = 2
 BOOLEAN_AND = 3
 BOOLEAN_XOR = 4
 
+# -------------- User interface ---------------------
+
+# The column order in the marker panel
+#
+ID_COLUMN = 0
+SESSION_COLUMN = 1
+LABEL_COLUMN = 2
+TARGET_COLUMN = 3
+X_COLUMN = 4
+Y_COLUMN = 5
+Z_COLUMN = 6
+
 #------------ Navigation defaults -------------------
 
+MARKER_COLOUR = (1.0, 1.0, 0.)
+MARKER_SIZE = 2
+ARROW_MARKER_SIZE = 10
+CALIBRATION_TRACKER_SAMPLES = 10
+FIDUCIAL_REGISTRATION_ERROR_THRESHOLD = 3.0
+
 SELECT = 0
 MTC = 1
 FASTRAK = 2
@@ -653,16 +682,21 @@ ISOTRAKII = 3
 PATRIOT = 4
 CAMERA = 5
 POLARIS = 6
-DEBUGTRACK = 7
-DISCTRACK = 8
+POLARISP4 = 7
+OPTITRACK = 8
+ROBOT = 9
+DEBUGTRACKRANDOM = 10
+DEBUGTRACKAPPROACH = 11
 DEFAULT_TRACKER = SELECT
 
 NDICOMPORT = b'COM1'
 
-TRACKER = [_("Select tracker:"), _("Claron MicronTracker"),
+TRACKERS = [_("Claron MicronTracker"),
            _("Polhemus FASTRAK"), _("Polhemus ISOTRAK II"),
            _("Polhemus PATRIOT"), _("Camera tracker"),
-           _("NDI Polaris"), _("Debug tracker"), _("Disconnect tracker")]
+           _("NDI Polaris"), _("NDI Polaris P4"),
+           _("Optitrack"), _("Robot tracker"),
+           _("Debug tracker (random)"), _("Debug tracker (approach)")]
 
 STATIC_REF = 0
 DYNAMIC_REF = 1
@@ -682,25 +716,57 @@ TR2 = wx.NewId()
 TR3 = wx.NewId()
 SET = wx.NewId()
 
-BTNS_IMG = {IR1: {0: _('LEI')},
-            IR2: {1: _('REI')},
-            IR3: {2: _('NAI')}}
+IMAGE_FIDUCIALS = [
+    {
+        'button_id': IR1,
+        'label': 'LEI',
+        'fiducial_name': 'LE',
+        'fiducial_index': 0,
+        'tip': _("Select left ear in image"),
+    },
+    {
+        'button_id': IR2,
+        'label': 'REI',
+        'fiducial_name': 'RE',
+        'fiducial_index': 1,
+        'tip': _("Select right ear in image"),
+    },
+    {
+        'button_id': IR3,
+        'label': 'NAI',
+        'fiducial_name': 'NA',
+        'fiducial_index': 2,
+        'tip': _("Select nasion in image"),
+    },
+]
 
-BTNS_IMG_MKS = {IR1: {0: 'LEI'},
-            IR2: {1: 'REI'},
-            IR3: {2: 'NAI'}}
+TRACKER_FIDUCIALS = [
+    {
+        'button_id': TR1,
+        'label': 'LET',
+        'fiducial_name': 'LE',
+        'fiducial_index': 0,
+        'tip': _("Select left ear with spatial tracker"),
+    },
+    {
+        'button_id': TR2,
+        'label': 'RET',
+        'fiducial_name': 'RE',
+        'fiducial_index': 1,
+        'tip': _("Select right ear with spatial tracker"),
+    },
+    {
+        'button_id': TR3,
+        'label': 'NAT',
+        'fiducial_name': 'NA',
+        'fiducial_index': 2,
+        'tip': _("Select nasion with spatial tracker"),
+    },
+]
 
-TIPS_IMG = [_("Select left ear in image"),
-            _("Select right ear in image"),
-            _("Select nasion in image")]
-
-BTNS_TRK = {TR1: {3: _('LET')},
-            TR2: {4: _('RET')},
-            TR3: {5: _('NAT')}}
-
-TIPS_TRK = [_("Select left ear with spatial tracker"),
-            _("Select right ear with spatial tracker"),
-            _("Select nasion with spatial tracker")]
+BTNS_IMG_MARKERS = {IR1: {0: 'LEI'},
+                    IR2: {1: 'REI'},
+                    IR3: {2: 'NAI'}}
 
 OBJL = wx.NewId()
 OBJR = wx.NewId()
@@ -708,28 +774,92 @@ OBJA = wx.NewId()
 OBJC = wx.NewId()
 OBJF = wx.NewId()
 
-BTNS_OBJ = {OBJL: {0: _('Left')},
-            OBJR: {1: _('Right')},
-            OBJA: {2: _('Anterior')},
-            OBJC: {3: _('Center')},
-            OBJF: {4: _('Fixed')}}
-
-TIPS_OBJ = [_("Select left object fiducial"),
-            _("Select right object fiducial"),
-            _("Select anterior object fiducial"),
-            _("Select object center"),
-            _("Attach sensor to object")]
+OBJECT_FIDUCIALS = [
+    {
+        'fiducial_index': 0,
+        'button_id': OBJL,
+        'label': _('Left'),
+        'tip': _("Select left object fiducial"),
+    },
+    {
+        'fiducial_index': 1,
+        'button_id': OBJR,
+        'label': _('Right'),
+        'tip': _("Select right object fiducial"),
+    },
+    {
+        'fiducial_index': 2,
+        'button_id': OBJA,
+        'label': _('Anterior'),
+        'tip': _("Select anterior object fiducial"),
+    },
+    {
+        'fiducial_index': 3,
+        'button_id': OBJC,
+        'label': _('Center'),
+        'tip': _("Select object center"),
+    },
+    {
+        'fiducial_index': 4,
+        'button_id': OBJF,
+        'label': _('Fixed'),
+        'tip': _("Attach sensor to object"),
+    },
+]
 
 MTC_PROBE_NAME = "1Probe"
 MTC_REF_NAME = "2Ref"
 MTC_OBJ_NAME = "3Coil"
 
-#OBJECT TRACKING
+# Object tracking
 ARROW_SCALE = 6
 ARROW_UPPER_LIMIT = 15
 #COIL_ANGLES_THRESHOLD = 3 * ARROW_SCALE
 COIL_ANGLES_THRESHOLD = 3
 COIL_COORD_THRESHOLD = 3
 TIMESTAMP = 2.0
+COIL_ANGLE_ARROW_PROJECTION_THRESHOLD = 5
 
 CAM_MODE = True
+
+# Tractography visualization
+N_TRACTS = 200
+PEEL_DEPTH = 10
+MAX_PEEL_DEPTH = 40
+SEED_OFFSET = 30
+SEED_RADIUS = 1.5
+
+# Increased the default sleep parameter from 0.1 to 0.15 to decrease CPU load during navigation.
+SLEEP_NAVIGATION = 0.2
+SLEEP_COORDINATES = 0.05
+
+BRAIN_OPACITY = 0.6
+N_CPU = psutil.cpu_count()
+# the max_sampling_step can be set to something different as well. Above 100 is probably not necessary
+TREKKER_CONFIG = {'seed_max': 1,
+                  'step_size': 0.03125,
+                  'min_fod': 0.05,
+                  'probe_quality': 3,
+                  'max_interval': 1,
+                  'min_radius_curvature': 0.625,
+                  'probe_length': 0.15625,
+                  'write_interval': 50,
+                  'numb_threads': '',
+                  'max_length': 250,
+                  'min_length': 10,
+                  'max_sampling_step': 100,
+                  'data_support_exponent': 0.5,
+                  'use_best_init': True,
+                  'init_max_est_trials': 100}
+
+MARKER_FILE_MAGICK_STRING = "##INVESALIUS3_MARKER_FILE_"
+CURRENT_MARKER_FILE_VERSION = 0
+WILDCARD_MARKER_FILES = _("Marker scanner coord files (*.mkss)|*.mkss") 
+
+# Serial port
+BAUD_RATES = [300, 1200, 2400, 4800, 9600, 19200, 38400, 57600, 115200]
+BAUD_RATE_DEFAULT_SELECTION = 4
+PULSE_DURATION_IN_MILLISECONDS = 0.2
+
+#Robot
+ROBOT_ElFIN_IP = ['143.107.220.251', '169.254.153.251', '127.0.0.1']
