@@ -1,12 +1,10 @@
 # -*- coding: UTF-8 -*-
 
-from six import with_metaclass
-
 import math
 import random
 import sys
 
-from wx.lib.pubsub import pub as Publisher
+from invesalius.pubsub import pub as Publisher
 
 import numpy as np
 import vtk
@@ -18,7 +16,7 @@ import invesalius.utils as utils
 
 from invesalius import math_utils
 from invesalius.gui.widgets.canvas_renderer import TextBox, CircleHandler, Ellipse, Polygon, CanvasHandlerBase
-from scipy.misc import imsave
+from imageio import imsave
 
 TYPE = {const.LINEAR: _(u"Linear"),
         const.ANGULAR: _(u"Angular"),
@@ -57,7 +55,7 @@ else:
 
 DEBUG_DENSITY = False
 
-class MeasureData(with_metaclass(utils.Singleton)):
+class MeasureData(metaclass=utils.Singleton):
     """
     Responsible to keep measures data.
     """
