@@ -38,7 +38,7 @@ import vtk
 import wx
 import wx.lib.agw.genericmessagedialog as GMD
 
-from wx.lib.pubsub import pub as Publisher
+from invesalius.pubsub import pub as Publisher
 
 if sys.platform == 'win32':
     try:
@@ -51,17 +51,18 @@ else:
 
 import invesalius.constants as const
 import invesalius.data.imagedata_utils as iu
+import invesalius.data.slice_ as sl
 import invesalius.data.polydata_utils as pu
 import invesalius.project as prj
 import invesalius.session as ses
 import invesalius.data.surface_process as surface_process
 import invesalius.utils as utl
-import invesalius.data.vtk_utils as vu
+import invesalius.data.vtk_utils as vtk_utils
 
 from invesalius.gui import dialogs
+from invesalius_cy import cy_mesh
 
-from invesalius.data import cy_mesh
-# TODO: Verificar ReleaseDataFlagOn and SetSource 
+# TODO: Verificar ReleaseDataFlagOn and SetSource
 
 
 class Surface():
@@ -115,14 +116,16 @@ class Surface():
         plist_filename = filename + u'.plist'
         #plist_filepath = os.path.join(dir_temp, filename + '.plist')
         temp_plist = tempfile.mktemp()
-        plistlib.writePlist(surface, temp_plist)
+        with open(temp_plist, 'w+b') as f:
+            plistlib.dump(surface, f)
 
         filelist[temp_plist] = plist_filename
 
         return plist_filename
 
     def OpenPList(self, filename):
-        sp = plistlib.readPlist(filename)
+        with open(filename, 'r+b') as f:
+            sp = plistlib.load(f, fmt=plistlib.FMT_XML)
         dirpath = os.path.abspath(os.path.split(filename)[0])
         self.index = sp['index']
         self.name = sp['name']
@@ -157,8 +160,7 @@ class SurfaceManager():
     def __init__(self):
         self.actors_dict = {}
         self.last_surface_index = 0
-        self.affine = None
-        self.converttoInV = None
+        self.convert2inv = None
         self.__bind_events()
 
         self._default_parameters = {
@@ -180,9 +182,6 @@ class SurfaceManager():
             session['surface'] = self._default_parameters
             session.WriteSessionFile()
 
-        print('Session', session)
-
-
     def __bind_events(self):
         Publisher.subscribe(self.AddNewActor, 'Create surface')
         Publisher.subscribe(self.SetActorTransparency,
@@ -208,8 +207,9 @@ class SurfaceManager():
 
         Publisher.subscribe(self.OnImportSurfaceFile, 'Import surface file')
 
-        Publisher.subscribe(self.UpdateAffineMatrix, 'Update affine matrix')
-        Publisher.subscribe(self.UpdateconverttoInVflag, 'Update converttoInV flag')
+        Publisher.subscribe(self.UpdateConvert2InvFlag, 'Update convert2inv flag')
+
+        Publisher.subscribe(self.CreateSurfaceFromPolydata, 'Create surface from polydata')
 
     def OnDuplicate(self, surface_indexes):
         proj = prj.Project()
@@ -337,35 +337,12 @@ class SurfaceManager():
             name = os.path.splitext(os.path.split(filename)[-1])[0]
             self.CreateSurfaceFromPolydata(polydata, name=name, scalar=scalar)
 
-    def UpdateAffineMatrix(self, affine, status):
-        try:
-            if status:
-                from numpy import hstack
-                from numpy.linalg import inv
-                affine = inv(affine)
-                affine[1, 3] = -affine[1, 3]
-                self.affine = hstack(affine)
-            else:
-                self.affine = None
-        except:
-            Publisher.sendMessage('Update affine matrix',
-                                  affine=None, status=False)
+    def UpdateConvert2InvFlag(self, convert2inv=False):
+        self.convert2inv = convert2inv
 
-    def UpdateconverttoInVflag(self, converttoInV):
-        self.converttoInV = converttoInV
-
-    def CreateSurfaceFromPolydata(self, polydata, overwrite=False,
-                                  name=None, colour=None,
-                                  transparency=None, volume=None, area=None, scalar=False):
-        if self.converttoInV and self.affine is not None:
-            transform = vtk.vtkTransform()
-            transform.SetMatrix(self.affine)
-            transformFilter = vtk.vtkTransformPolyDataFilter()
-            transformFilter.SetTransform(transform)
-            transformFilter.SetInputData(polydata)
-            transformFilter.Update()
-            polydata = transformFilter.GetOutput()
-            self.converttoInV = None
+    def CreateSurfaceFromPolydata(self, polydata, overwrite=False, index=None,
+                                  name=None, colour=None, transparency=None,
+                                  volume=None, area=None, scalar=False):
 
         normals = vtk.vtkPolyDataNormals()
         normals.SetInputData(polydata)
@@ -379,15 +356,31 @@ class SurfaceManager():
             mapper.ScalarVisibilityOn()
         else:
             mapper.ScalarVisibilityOff()
-        mapper.ImmediateModeRenderingOn() # improve performance
+        #  mapper.ImmediateModeRenderingOn() # improve performance
 
         actor = vtk.vtkActor()
         actor.SetMapper(mapper)
+        actor.GetProperty().SetBackfaceCulling(1)
 
-        print("BOunds", actor.GetBounds())
+        if self.convert2inv:
+            # convert between invesalius and world space with shift in the Y coordinate
+            affine = sl.Slice().affine
+            # TODO: Check that this is needed with the new way of using affine
+            #  now the affine should be at least the identity(4) and never None
+            if affine is not None:
+                matrix_shape = sl.Slice().matrix.shape
+                spacing = sl.Slice().spacing
+                img_shift = spacing[1] * (matrix_shape[1] - 1)
+                affine = sl.Slice().affine.copy()
+                affine[1, -1] -= img_shift
+                affine_vtk = vtk_utils.numpy_to_vtkMatrix4x4(affine)
+
+                actor.SetUserMatrix(affine_vtk)
 
         if overwrite:
-            surface = Surface(index = self.last_surface_index)
+            if index is None:
+                index = self.last_surface_index
+            surface = Surface(index=index)
         else:
             surface = Surface()
 
@@ -415,6 +408,14 @@ class SurfaceManager():
         # Set actor colour and transparency
         actor.GetProperty().SetColor(surface.colour)
         actor.GetProperty().SetOpacity(1-surface.transparency)
+
+        if overwrite and self.actors_dict.keys():
+            try:
+                old_actor = self.actors_dict[index]
+                Publisher.sendMessage('Remove surface actor from viewer', actor=old_actor)
+            except KeyError:
+                pass
+
         self.actors_dict[surface.index] = actor
 
         session = ses.Session()
@@ -441,7 +442,6 @@ class SurfaceManager():
         self.last_surface_index = surface.index
 
         Publisher.sendMessage('Load surface actor into viewer', actor=actor)
-
         Publisher.sendMessage('Update surface info in GUI', surface=surface)
         return surface.index
 
@@ -457,8 +457,9 @@ class SurfaceManager():
         # restarting the surface index
         Surface.general_index = -1
 
-        Publisher.sendMessage('Update affine matrix',
-                              affine=None, status=False)
+        self.affine_vtk = None
+        self.convert2inv = False
+
 
     def OnSelectSurface(self, surface_index):
         #self.last_surface_index = surface_index
@@ -490,10 +491,11 @@ class SurfaceManager():
             mapper = vtk.vtkPolyDataMapper()
             mapper.SetInputConnection(stripper.GetOutputPort())
             mapper.ScalarVisibilityOff()
-            mapper.ImmediateModeRenderingOn() # improve performance
+            #  mapper.ImmediateModeRenderingOn() # improve performance
 
             # Represent an object (geometry & properties) in the rendered scene
             actor = vtk.vtkActor()
+            actor.GetProperty().SetBackfaceCulling(1)
             actor.SetMapper(mapper)
 
             # Set actor colour and transparency
@@ -532,10 +534,11 @@ class SurfaceManager():
         mapper.SetInputData(polydata)
         mapper.ScalarVisibilityOff()
         #  mapper.ReleaseDataFlagOn()
-        mapper.ImmediateModeRenderingOn() # improve performance
+        #  mapper.ImmediateModeRenderingOn() # improve performance
 
         # Represent an object (geometry & properties) in the rendered scene
         actor = vtk.vtkActor()
+        actor.GetProperty().SetBackfaceCulling(1)
         actor.SetMapper(mapper)
         del mapper
         #Create Surface instance
@@ -593,11 +596,18 @@ class SurfaceManager():
         """
         Create surface actor, save into project and send it to viewer.
         """
+        if mask.matrix.max() < 127:
+            wx.MessageBox(_("It's not possible to create a surface because there is not any voxel selected on mask"), _("Create surface warning"))
+            return
         t_init = time.time()
         matrix = slice_.matrix
         filename_img = slice_.matrix_filename
         spacing = slice_.spacing
 
+        mask_temp_file = mask.temp_file
+        mask_shape = mask.matrix.shape
+        mask_dtype = mask.matrix.dtype
+
         algorithm = surface_parameters['method']['algorithm']
         options = surface_parameters['method']['options']
 
@@ -608,6 +618,8 @@ class SurfaceManager():
 
         fill_border_holes = surface_parameters['options'].get('fill_border_holes', True)
 
+        print(surface_parameters)
+
         mode = 'CONTOUR' # 'GRAYSCALE'
         min_value, max_value = mask.threshold_range
         colour = mask.colour[:3]
@@ -641,6 +653,16 @@ class SurfaceManager():
         else:
             flip_image = True
 
+        if imagedata_resolution > 0:
+            spacing = tuple([s * imagedata_resolution for s in spacing])
+            matrix = iu.resize_image_array(matrix, 1.0/imagedata_resolution, True)
+            mask = iu.resize_image_array(mask.matrix, 1.0/imagedata_resolution, True)
+
+            filename_img = matrix.filename
+            mask_temp_file = mask.filename
+            mask_shape = mask.shape
+            mask_dtype = mask.dtype
+
         n_processors = multiprocessing.cpu_count()
 
         o_piece = 1
@@ -649,10 +671,13 @@ class SurfaceManager():
         n_pieces = int(round(matrix.shape[0] / piece_size + 0.5, 0))
 
         filenames = []
-        pool = multiprocessing.Pool(processes=min(n_pieces, n_processors))
+        ctx = multiprocessing.get_context('spawn')
+        pool = ctx.Pool(processes=min(n_pieces, n_processors))
         manager = multiprocessing.Manager()
         msg_queue = manager.Queue(1)
 
+        print("Resolution", imagedata_resolution)
+
         # If InVesalius is running without GUI
         if wx.GetApp() is None:
             for i in range(n_pieces):
@@ -662,8 +687,8 @@ class SurfaceManager():
                 print("new_piece", roi)
                 f = pool.apply_async(surface_process.create_surface_piece,
                                      args = (filename_img, matrix.shape, matrix.dtype,
-                                             mask.temp_file, mask.matrix.shape,
-                                             mask.matrix.dtype, roi, spacing, mode,
+                                             mask_temp_file, mask_shape,
+                                             mask_dtype, roi, spacing, mode,
                                              min_value, max_value, decimate_reduction,
                                              smooth_relaxation_factor,
                                              smooth_iterations, language, flip_image,
@@ -720,31 +745,18 @@ class SurfaceManager():
                 end = init + piece_size + o_piece
                 roi = slice(init, end)
                 print("new_piece", roi)
-                try:
-                    f = pool.apply_async(surface_process.create_surface_piece,
-                                         args = (filename_img, matrix.shape, matrix.dtype,
-                                                 mask.temp_file, mask.matrix.shape,
-                                                 mask.matrix.dtype, roi, spacing, mode,
-                                                 min_value, max_value, decimate_reduction,
-                                                 smooth_relaxation_factor,
-                                                 smooth_iterations, language, flip_image,
-                                                 algorithm != 'Default', algorithm,
-                                                 imagedata_resolution, fill_border_holes),
-                                         callback=lambda x: filenames.append(x),
-                                         error_callback=functools.partial(self._on_callback_error,
-                                                                          dialog=sp))
-                # python2
-                except TypeError:
-                    f = pool.apply_async(surface_process.create_surface_piece,
-                                         args = (filename_img, matrix.shape, matrix.dtype,
-                                                 mask.temp_file, mask.matrix.shape,
-                                                 mask.matrix.dtype, roi, spacing, mode,
-                                                 min_value, max_value, decimate_reduction,
-                                                 smooth_relaxation_factor,
-                                                 smooth_iterations, language, flip_image,
-                                                 algorithm != 'Default', algorithm,
-                                                 imagedata_resolution, fill_border_holes),
-                                         callback=lambda x: filenames.append(x))
+                f = pool.apply_async(surface_process.create_surface_piece,
+                                     args = (filename_img, matrix.shape, matrix.dtype,
+                                             mask_temp_file, mask_shape,
+                                             mask_dtype, roi, spacing, mode,
+                                             min_value, max_value, decimate_reduction,
+                                             smooth_relaxation_factor,
+                                             smooth_iterations, language, flip_image,
+                                             algorithm != 'Default', algorithm,
+                                             imagedata_resolution, fill_border_holes),
+                                     callback=lambda x: filenames.append(x),
+                                     error_callback=functools.partial(self._on_callback_error,
+                                                                      dialog=sp))
 
             while len(filenames) != n_pieces:
                 if sp.WasCancelled() or not sp.running:
@@ -871,9 +883,31 @@ class SurfaceManager():
                 os.remove(_temp_file)
 
             temp_file = utl.decode(temp_file, const.FS_ENCODE)
-            self._export_surface(temp_file, filetype)
+            try:
+                self._export_surface(temp_file, filetype)
+            except ValueError:
+                if wx.GetApp() is None:
+                    print("It was not possible to export the surface because the surface is empty")
+                else:
+                    wx.MessageBox(_("It was not possible to export the surface because the surface is empty"), _("Export surface error"))
+                return
+
+            try:
+                shutil.move(temp_file, filename)
+            except PermissionError as err:
+                dirpath = os.path.split(filename)[0]
+                if wx.GetApp() is None:
+                    print(_("It was not possible to export the surface because you don't have permission to write to {} folder: {}".format(dirpath, err)))
+                else:
+                    dlg = dialogs.ErrorMessageBox(
+                        None,
+                        _("Export surface error"),
+                        "It was not possible to export the surface because you don't have permission to write to {}:\n{}".format(dirpath, err)
+                    )
+                    dlg.ShowModal()
+                    dlg.Destroy()
+                os.remove(temp_file)
 
-            shutil.move(temp_file, filename)
 
     def _export_surface(self, filename, filetype):
         if filetype in (const.FILETYPE_STL,
@@ -898,6 +932,9 @@ class SurfaceManager():
             else:
                 polydata = pu.Merge(polydata_list)
 
+            if polydata.GetNumberOfPoints() == 0:
+                raise ValueError
+
             # Having a polydata that represents all surfaces
             # selected, we write it, according to filetype
             if filetype == const.FILETYPE_STL:
