@@ -18,8 +18,10 @@
 #--------------------------------------------------------------------------
 import invesalius.constants as const
 import invesalius.gui.dialogs as dlg
+from invesalius.pubsub import pub as Publisher
 # TODO: Disconnect tracker when a new one is connected
 # TODO: Test if there are too many prints when connection fails
+# TODO: Redesign error messages. No point in having "Could not connect to default tracker" in all trackers
 
 
 def TrackerConnection(tracker_id, trck_init, action):
@@ -39,7 +41,11 @@ def TrackerConnection(tracker_id, trck_init, action):
                     const.PATRIOT: PolhemusTracker,    # PATRIOT
                     const.CAMERA: CameraTracker,      # CAMERA
                     const.POLARIS: PolarisTracker,      # POLARIS
-                    const.DEBUGTRACK: DebugTracker}
+                    const.POLARISP4: PolarisP4Tracker,  # POLARISP4
+                    const.OPTITRACK: OptitrackTracker, #Optitrack
+                    const.ROBOT: RobotTracker,   #Robot
+                    const.DEBUGTRACKRANDOM: DebugTrackerRandom,
+                    const.DEBUGTRACKAPPROACH: DebugTrackerApproach}
 
         trck_init = trck_fcn[tracker_id](tracker_id)
 
@@ -61,49 +67,6 @@ def DefaultTracker(tracker_id):
     # return tracker initialization variable and type of connection
     return trck_init, 'wrapper'
 
-def PolarisTracker(tracker_id):
-    from wx import ID_OK
-    trck_init = None
-    dlg_port = dlg.SetNDIconfigs()
-    if dlg_port.ShowModal() == ID_OK:
-        com_port, PROBE_DIR, REF_DIR, OBJ_DIR = dlg_port.GetValue()
-        try:
-            import pypolaris
-            lib_mode = 'wrapper'
-            trck_init = pypolaris.pypolaris()
-
-            if trck_init.Initialize(com_port, PROBE_DIR, REF_DIR, OBJ_DIR) != 0:
-                trck_init = None
-                lib_mode = None
-                print('Could not connect to default tracker.')
-            else:
-                print('Connect to polaris tracking device.')
-
-        except:
-            lib_mode = 'error'
-            trck_init = None
-            print('Could not connect to default tracker.')
-    else:
-        lib_mode = None
-        print('Could not connect to default tracker.')
-
-    # return tracker initialization variable and type of connection
-    return trck_init, lib_mode
-
-def CameraTracker(tracker_id):
-    trck_init = None
-    try:
-        import invesalius.data.camera_tracker as cam
-        trck_init = cam.camera()
-        trck_init.Initialize()
-        print('Connect to camera tracking device.')
-
-    except:
-        print('Could not connect to default tracker.')
-
-    # return tracker initialization variable and type of connection
-    return trck_init, 'wrapper'
-
 def ClaronTracker(tracker_id):
     import invesalius.constants as const
     from invesalius import inv_paths
@@ -135,7 +98,6 @@ def ClaronTracker(tracker_id):
 
     return trck_init, lib_mode
 
-
 def PolhemusTracker(tracker_id):
     try:
         trck_init = PlhWrapperConnection(tracker_id)
@@ -151,14 +113,144 @@ def PolhemusTracker(tracker_id):
     except:
         trck_init = None
         lib_mode = 'error'
-        print('Could not connect to Polhemus.')
+        print('Could not connect to Polhemus by any method.')
 
     return trck_init, lib_mode
 
+def CameraTracker(tracker_id):
+    trck_init = None
+    try:
+        import invesalius.data.camera_tracker as cam
+        trck_init = cam.camera()
+        trck_init.Initialize()
+        print('Connect to camera tracking device.')
 
-def DebugTracker(tracker_id):
+    except:
+        print('Could not connect to camera tracker.')
+
+    # return tracker initialization variable and type of connection
+    return trck_init, 'wrapper'
+
+def PolarisTracker(tracker_id):
+    from wx import ID_OK
+    trck_init = None
+    dlg_port = dlg.SetNDIconfigs()
+    if dlg_port.ShowModal() == ID_OK:
+        com_port, PROBE_DIR, REF_DIR, OBJ_DIR = dlg_port.GetValue()
+        try:
+            import pypolaris
+            lib_mode = 'wrapper'
+            trck_init = pypolaris.pypolaris()
+
+            if trck_init.Initialize(com_port, PROBE_DIR, REF_DIR, OBJ_DIR) != 0:
+                trck_init = None
+                lib_mode = None
+                print('Could not connect to polaris tracker.')
+            else:
+                print('Connect to polaris tracking device.')
+
+        except:
+            lib_mode = 'error'
+            trck_init = None
+            print('Could not connect to polaris tracker.')
+    else:
+        lib_mode = None
+        print('Could not connect to polaris tracker.')
+
+    # return tracker initialization variable and type of connection
+    return trck_init, lib_mode
+
+def PolarisP4Tracker(tracker_id):
+    from wx import ID_OK
+    trck_init = None
+    dlg_port = dlg.SetNDIconfigs()
+    if dlg_port.ShowModal() == ID_OK:
+        com_port, PROBE_DIR, REF_DIR, OBJ_DIR = dlg_port.GetValue()
+        try:
+            import pypolarisP4
+            lib_mode = 'wrapper'
+            trck_init = pypolarisP4.pypolarisP4()
+
+            if trck_init.Initialize(com_port, PROBE_DIR, REF_DIR, OBJ_DIR) != 0:
+                trck_init = None
+                lib_mode = None
+                print('Could not connect to Polaris P4 tracker.')
+            else:
+                print('Connect to Polaris P4 tracking device.')
+
+        except:
+            lib_mode = 'error'
+            trck_init = None
+            print('Could not connect to Polaris P4 tracker.')
+    else:
+        lib_mode = None
+        print('Could not connect to Polaris P4 tracker.')
+    # return tracker initialization variable and type of connection
+    return trck_init, lib_mode
+
+def OptitrackTracker(tracker_id):
+    """
+    Imports optitrack wrapper from Motive 2.2. Initialize cameras, attach listener, loads Calibration, loads User Profile
+    (Rigid bodies information).
+
+    Parameters
+    ----------
+    tracker_id : Optitrack ID
+
+    Returns
+    -------
+    trck_init : local name for Optitrack module
+    """
+    from wx import ID_OK
+    trck_init = None
+    dlg_port = dlg.SetOptitrackconfigs()
+    if dlg_port.ShowModal() == ID_OK:
+        Cal_optitrack, User_profile_optitrack = dlg_port.GetValue()
+        try:
+            import optitrack
+            trck_init = optitrack.optr()
+
+            if trck_init.Initialize(Cal_optitrack, User_profile_optitrack)==0:
+                trck_init.Run() #Runs once Run function, to update cameras.
+                lib_mode = 'wrapper'
+            else:
+                trck_init = None
+                lib_mode = 'error'
+        except ImportError:
+            lib_mode = 'error'
+            print('Error')
+    else:
+        lib_mode = None
+        print('#####')
+    return trck_init, lib_mode
+
+def RobotTracker(tracker_id):
+    from wx import ID_OK
+
+    dlg_device = dlg.SetTrackerDeviceToRobot()
+    if dlg_device.ShowModal() == ID_OK:
+        tracker_id = dlg_device.GetValue()
+        if tracker_id:
+            trck_init = TrackerConnection(tracker_id, None, 'connect')
+            if trck_init[0]:
+                dlg_ip = dlg.SetRobotIP()
+                if dlg_ip.ShowModal() == ID_OK:
+                    robot_IP = dlg_ip.GetValue()
+                    Publisher.sendMessage('Connect to robot', robot_IP=robot_IP)
+
+                    return trck_init, tracker_id
+
+    return None, None
+
+def DebugTrackerRandom(tracker_id):
     trck_init = True
-    print('Debug device started.')
+    print('Debug device (random) started.')
+    return trck_init, 'debug'
+
+
+def DebugTrackerApproach(tracker_id):
+    trck_init = True
+    print('Debug device (approach) started.')
     return trck_init, 'debug'
 
 
@@ -181,10 +273,10 @@ def PlhWrapperConnection(tracker_id):
                 sleep(0.175)
         else:
             trck_init = None
-            print('Could not connect to Polhemus via wrapper without error.')
+            print('Could not connect to Polhemus via wrapper without error: Initialize is False.')
     except:
         trck_init = None
-        print('Could not connect to Polhemus via wrapper with error.')
+        print('Could not connect to Polhemus via wrapper without error: Import failed.')
 
     return trck_init
 
@@ -193,11 +285,13 @@ def PlhSerialConnection(tracker_id):
     import serial
     from wx import ID_OK
     trck_init = None
-    dlg_port = dlg.SetCOMport()
+    dlg_port = dlg.SetCOMPort(select_baud_rate=False)
     if dlg_port.ShowModal() == ID_OK:
-        com_port = dlg_port.GetValue()
+        com_port = dlg_port.GetCOMPort()
+        baud_rate = 115200
+
         try:
-            trck_init = serial.Serial(com_port, baudrate=115200, timeout=0.03)
+            trck_init = serial.Serial(com_port, baudrate=baud_rate, timeout=0.03)
 
             if tracker_id == 2:
                 # Polhemus FASTRAK needs configurations first
@@ -219,10 +313,10 @@ def PlhSerialConnection(tracker_id):
         except:
             lib_mode = 'error'
             trck_init = None
-            print('Could not connect to default tracker.')
+            print('Could not connect to Polhemus tracker.')
     else:
         lib_mode = None
-        print('Could not connect to default tracker.')
+        print('Could not connect to Polhemus tracker.')
 
     return trck_init
 
@@ -260,7 +354,6 @@ def PlhUSBConnection(tracker_id):
 
     return trck_init
 
-
 def DisconnectTracker(tracker_id, trck_init):
     """
     Disconnect current spatial tracker
@@ -269,7 +362,7 @@ def DisconnectTracker(tracker_id, trck_init):
     :param trck_init: Initialization variable of tracking device.
     """
 
-    if tracker_id == const.DEBUGTRACK:
+    if tracker_id == const.DEBUGTRACKRANDOM or tracker_id == const.DEBUGTRACKAPPROACH:
         trck_init = False
         lib_mode = 'debug'
         print('Debug tracker disconnected.')
@@ -280,6 +373,13 @@ def DisconnectTracker(tracker_id, trck_init):
                 trck_init = False
                 lib_mode = 'serial'
                 print('Tracker disconnected.')
+            elif tracker_id == const.ROBOT:
+                Publisher.sendMessage('Reset robot', data=None)
+                if trck_init[1] == const.DEBUGTRACKRANDOM or trck_init[1] == const.DEBUGTRACKAPPROACH:
+                    trck_init[0].Close()
+                trck_init = False
+                lib_mode = 'wrapper'
+                print('Tracker disconnected.')
             else:
                 trck_init.Close()
                 trck_init = False
