@@ -31,7 +31,7 @@ from vtk.wx.wxVTKRenderWindowInteractor import wxVTKRenderWindowInteractor
 import invesalius.data.styles as styles
 import wx
 import sys
-from wx.lib.pubsub import pub as Publisher
+from invesalius.pubsub import pub as Publisher
 
 try:
     from agw import floatspin as FS
@@ -102,13 +102,13 @@ class ContourMIPConfig(wx.Panel):
         self.txt_mip_border = wx.StaticText(self, -1, _("Sharpness"))
 
         sizer = wx.BoxSizer(wx.HORIZONTAL)
-        sizer.Add(txt_mip_size, 0, wx.EXPAND | wx.ALL, 2)
+        sizer.Add(txt_mip_size, 0, wx.ALIGN_CENTER_VERTICAL | wx.ALL, 2)
         sizer.Add(self.mip_size_spin, 0)
         try:
             sizer.Add(10, 0)
         except TypeError:
             sizer.Add((10, 0))
-        sizer.Add(self.txt_mip_border, 0, wx.EXPAND | wx.ALL, 2)
+        sizer.Add(self.txt_mip_border, 0, wx.ALIGN_CENTER_VERTICAL | wx.ALL, 2)
         sizer.Add(self.border_spin, 0, wx.EXPAND)
         try:
             sizer.Add(10, 0)
@@ -232,11 +232,11 @@ class Viewer(wx.Panel):
         self.mip_ctrls.Hide()
 
         sizer = wx.BoxSizer(wx.HORIZONTAL)
-        sizer.Add(self.interactor, 1, wx.EXPAND|wx.GROW)
+        sizer.Add(self.interactor, 1, wx.EXPAND)
         sizer.Add(scroll, 0, wx.EXPAND|wx.GROW)
 
         background_sizer = wx.BoxSizer(wx.VERTICAL)
-        background_sizer.Add(sizer, 1, wx.EXPAND|wx.GROW|wx.ALL, 2)
+        background_sizer.Add(sizer, 1, wx.EXPAND)
         #background_sizer.Add(self.mip_ctrls, 0, wx.EXPAND|wx.GROW|wx.ALL, 2)
         self.SetSizer(background_sizer)
         background_sizer.Fit(self)
@@ -303,8 +303,8 @@ class Viewer(wx.Panel):
             self.style.CleanUp()
 
         del self.style
-        
-        style = styles.get_style(state)(self)
+
+        style = styles.Styles.get_style(state)(self)
 
         setup = getattr(style, 'SetUp', None)
         if setup:
@@ -565,15 +565,31 @@ class Viewer(wx.Panel):
         if self.slice_data.cursor:
             self.slice_data.cursor.SetColour(colour_vtk)
 
-    def UpdateSlicesNavigation(self, arg, position):
+    def UpdateSlicesPosition(self, position):
         # Get point from base change
-        ux, uy, uz = position[:3]
-        px, py = self.get_slice_pixel_coord_by_world_pos(ux, uy, uz)
+        px, py = self.get_slice_pixel_coord_by_world_pos(*position)
         coord = self.calcultate_scroll_position(px, py)
+        # Debugging coordinates. For a 1.0 spacing axis the coord and position is the same,
+        # but for a spacing dimension =! 1, the coord and position are different
+        # print("\nPosition: {}".format(position))
+        # print("Scroll position: {}".format(coord))
+        # print("Slice actor bounds: {}".format(self.slice_data.actor.GetBounds()))
+        # print("Scroll from int of position: {}\n".format([round(s) for s in position]))
 
-        self.cross.SetFocalPoint((ux, uy, uz))
+        # this call did not affect the working code
+        # self.cross.SetFocalPoint(coord)
+
+        # update the image slices in all three orientations
         self.ScrollSlice(coord)
-        Publisher.sendMessage('Set ball reference position', position=(ux, uy, uz))
+
+    def SetCrossFocalPoint(self, position):
+        """
+        Sets the cross focal point for all slice panels (axial, coronal, sagittal). This function is also called via
+        pubsub messaging and may receive a list of 6 coordinates. Thus, limiting the number of list elements in the
+        SetFocalPoint call is required.
+        :param position: list of 6 coordinates in vtk world coordinate system wx, wy, wz
+        """
+        self.cross.SetFocalPoint(position[:3])
 
     def ScrollSlice(self, coord):
         if self.orientation == "AXIAL":
@@ -634,6 +650,27 @@ class Viewer(wx.Panel):
             my = round((z - zi)/self.slice_.spacing[2], 0)
         return int(mx), int(my)
 
+    def get_vtk_mouse_position(self):
+        """
+        Get Mouse position inside a wxVTKRenderWindowInteractorself. Return a
+        tuple with X and Y position.
+        Please use this instead of using iren.GetEventPosition because it's
+        not returning the correct values on Mac with HighDPI display, maybe
+        the same is happing with Windows and Linux, we need to test.
+        """
+        mposx, mposy = wx.GetMousePosition()
+        cposx, cposy = self.interactor.ScreenToClient((mposx, mposy))
+        mx, my = cposx, self.interactor.GetSize()[1] - cposy
+        if sys.platform == 'darwin':
+            # It's needed to mutiple by scale factor in HighDPI because of
+            # https://docs.wxpython.org/wx.glcanvas.GLCanvas.html
+            # For now we are doing this only on Mac but it may be needed on
+            # Windows and Linux too.
+            scale = self.interactor.GetContentScaleFactor()
+            mx *= scale
+            my *= scale
+        return int(mx), int(my)
+
     def get_coordinate_cursor(self, mx, my, picker=None):
         """
         Given the mx, my screen position returns the x, y, z position in world
@@ -816,10 +853,12 @@ class Viewer(wx.Panel):
         Publisher.subscribe(self.ChangeSliceNumber,
                                  ('Set scroll position',
                                   self.orientation))
-        Publisher.subscribe(self.__update_cross_position,
-                                'Update cross position')
-        Publisher.subscribe(self.UpdateSlicesNavigation,
-                            'Co-registered points')
+        # Publisher.subscribe(self.__update_cross_position,
+        #                     'Update cross position')
+        # Publisher.subscribe(self.__update_cross_position,
+        #                     'Update cross position %s' % self.orientation)
+        Publisher.subscribe(self.SetCrossFocalPoint, 'Set cross focal point')
+        Publisher.subscribe(self.UpdateSlicesPosition, 'Update slices position')
         ###
         #  Publisher.subscribe(self.ChangeBrushColour,
                                  #  'Add mask')
@@ -830,9 +869,9 @@ class Viewer(wx.Panel):
         Publisher.subscribe(self.UpdateWindowLevelText,
                             'Update window level text')
 
-        #Publisher.subscribe(self._set_cross_visibility,\
-        #                         'Set cross visibility')
-        ###
+        Publisher.subscribe(self._set_cross_visibility,
+                                 'Set cross visibility')
+
         Publisher.subscribe(self.__set_layout,
                                 'Set slice viewer layout')
 
@@ -917,8 +956,6 @@ class Viewer(wx.Panel):
 
     def _export_picture(self, id, filename, filetype):
         view_prop_list = []
-        view_prop_list.append(self.slice_data.box_actor)
-        self.slice_data.renderer.RemoveViewProp(self.slice_data.box_actor)
 
         dict = {"AXIAL": const.AXIAL,
                 "CORONAL": const.CORONAL,
@@ -1012,7 +1049,6 @@ class Viewer(wx.Panel):
         #self.scroll.Bind(wx.EVT_SCROLL_ENDSCROLL, self.OnScrollBarRelease)
         self.interactor.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
         self.interactor.Bind(wx.EVT_RIGHT_UP, self.OnContextMenu)
-        self.interactor.Bind(wx.EVT_SIZE, self.OnSize)
 
     def LoadImagedata(self, mask_dict):
         self.SetInput(mask_dict)
@@ -1053,7 +1089,7 @@ class Viewer(wx.Panel):
                 x, y = const.TEXT_POS_LEFT_DOWN
                 slice_data.text.SetPosition((x+slice_xi,y+slice_yi))
                 slice_data.SetCursor(self.__create_cursor())
-                slice_data.SetSize((w, h))
+                # slice_data.SetSize((w, h))
                 self.__update_camera(slice_data)
 
                 style = 0
@@ -1067,7 +1103,7 @@ class Viewer(wx.Panel):
                 if i == self.layout[0] - 1:
                     style = style | sd.BORDER_RIGHT
 
-                slice_data.SetBorderStyle(style)
+                # slice_data.SetBorderStyle(style)
                 n += 1
 
     def __create_cursor(self):
@@ -1093,7 +1129,7 @@ class Viewer(wx.Panel):
         self.__build_cross_lines()
 
         self.canvas = CanvasRendererCTX(self, self.slice_data.renderer, self.slice_data.canvas_renderer, self.orientation)
-        self.canvas.draw_list.append(self.slice_data.text)
+        self.canvas.draw_list.append(self.slice_data)
 
         # Set the slice number to the last slice to ensure the camera if far
         # enough to show all slices.
@@ -1136,8 +1172,9 @@ class Viewer(wx.Panel):
 
         renderer.AddActor(cross_actor)
 
-    def __update_cross_position(self, position):
-        self.cross.SetFocalPoint(position)
+    # def __update_cross_position(self, arg, position):
+    #     # self.cross.SetFocalPoint(position[:3])
+    #     self.UpdateSlicesPosition(None, position)
 
     def _set_cross_visibility(self, visibility):
         self.cross_actor.SetVisibility(visibility)
@@ -1188,10 +1225,10 @@ class Viewer(wx.Panel):
         slice_data.canvas_renderer = canvas_renderer
         slice_data.overlay_renderer = overlay_renderer
         slice_data.actor = actor
-        slice_data.SetBorderStyle(sd.BORDER_ALL)
+        # slice_data.SetBorderStyle(sd.BORDER_ALL)
         renderer.AddActor(actor)
         #  renderer.AddActor(slice_data.text.actor)
-        renderer.AddViewProp(slice_data.box_actor)
+        # renderer.AddViewProp(slice_data.box_actor)
 
         return slice_data
 
@@ -1293,14 +1330,17 @@ class Viewer(wx.Panel):
         self.set_slice_number(pos)
         if update3D:
             self.UpdateSlice3D(pos)
-        if self.state == const.SLICE_STATE_CROSS:
-            # Update other slice's cross according to the new focal point from
-            # the actual orientation.
-            focal_point = self.cross.GetFocalPoint()
-            Publisher.sendMessage('Update cross position', position=focal_point)
-            Publisher.sendMessage('Update slice viewer')
-        else:
-            self.interactor.Render()
+
+        # This Render needs to come before the self.style.OnScrollBar, otherwise the GetFocalPoint will sometimes
+        # provide the non-updated coordinate and the cross focal point will lag one pixel behind the actual
+        # scroll position
+        self.interactor.Render()
+
+        try:
+            self.style.OnScrollBar()
+        except AttributeError:
+            pass
+
         if evt:
             if self._flush_buffer:
                 self.slice_.apply_slice_buffer_to_mask(self.orientation)
@@ -1331,10 +1371,12 @@ class Viewer(wx.Panel):
         if (evt.GetKeyCode() == wx.WXK_UP and pos > min):
             self.OnScrollForward()
             self.OnScrollBar()
+            skip = False
 
         elif (evt.GetKeyCode() == wx.WXK_DOWN and pos < max):
             self.OnScrollBackward()
             self.OnScrollBar()
+            skip = False
 
         elif (evt.GetKeyCode() == wx.WXK_NUMPAD_ADD):
             actual_value = self.mip_ctrls.mip_size_spin.GetValue()
@@ -1342,6 +1384,7 @@ class Viewer(wx.Panel):
             if self.mip_ctrls.mip_size_spin.GetValue() != actual_value:
                 self.number_slices = self.mip_ctrls.mip_size_spin.GetValue()
                 self.ReloadActualSlice()
+            skip = False
 
         elif (evt.GetKeyCode() == wx.WXK_NUMPAD_SUBTRACT):
             actual_value = self.mip_ctrls.mip_size_spin.GetValue()
@@ -1349,6 +1392,7 @@ class Viewer(wx.Panel):
             if self.mip_ctrls.mip_size_spin.GetValue() != actual_value:
                 self.number_slices = self.mip_ctrls.mip_size_spin.GetValue()
                 self.ReloadActualSlice()
+            skip = False
 
         elif evt.GetKeyCode() in projections:
             self.slice_.SetTypeProjection(projections[evt.GetKeyCode()])
@@ -1388,14 +1432,6 @@ class Viewer(wx.Panel):
             self.scroll.SetThumbPosition(pos)
             self.OnScrollBar()
 
-    def OnSize(self, evt):
-        w, h = evt.GetSize()
-        w = float(w)
-        h = float(h)
-        if self.slice_data:
-            self.slice_data.SetSize((w, h))
-        evt.Skip()
-
     def OnSetMIPSize(self, number_slices):
         self.number_slices = number_slices
         self.ReloadActualSlice()
@@ -1427,10 +1463,19 @@ class Viewer(wx.Panel):
         self.overwrite_mask = flag
 
     def set_slice_number(self, index):
+        max_slice_number = sl.Slice().GetNumberOfSlices(self.orientation)
+        if index < 0:
+            index = 0
+        if index >= max_slice_number:
+            index = max_slice_number - 1
         inverted = self.mip_ctrls.inverted.GetValue()
         border_size = self.mip_ctrls.border_spin.GetValue()
-        image = self.slice_.GetSlices(self.orientation, index,
-                                      self.number_slices, inverted, border_size)
+        try:
+            image = self.slice_.GetSlices(self.orientation, index,
+                                          self.number_slices, inverted,
+                                          border_size)
+        except IndexError:
+            return
         self.slice_data.actor.SetInputData(image)
         for actor in self.actors_by_slice_number[self.slice_data.number]:
             self.slice_data.renderer.RemoveActor(actor)
@@ -1496,7 +1541,6 @@ class Viewer(wx.Panel):
 
     def UpdateCross(self, coord):
         self.cross.SetFocalPoint(coord)
-        Publisher.sendMessage('Set ball reference position', position=self.cross.GetFocalPoint())
         Publisher.sendMessage('Co-registered points',  arg=None, position=(coord[0], coord[1], coord[2], 0., 0., 0.))
         self.OnScrollBar()
         self.interactor.Render()
@@ -1530,3 +1574,39 @@ class Viewer(wx.Panel):
                 renderer.RemoveActor(actor)
                 # and remove the actor from the actor's list
                 self.actors_by_slice_number[slice_number].remove(actor)
+
+    def get_actual_mask(self):
+        # Returns actual mask. Returns None if there is not a mask or no mask
+        # visible.
+        mask = self.slice_.current_mask
+        return mask
+
+    def get_slice(self):
+        return self.slice_
+
+    def discard_slice_cache(self, all_orientations=False, vtk_cache=True):
+        if all_orientations:
+            for orientation in self.slice_.buffer_slices:
+                buffer_ = self.slice_.buffer_slices[orientation]
+                buffer_.discard_image()
+                if vtk_cache:
+                    buffer_.discard_vtk_image()
+        else:
+            buffer_ = self.slice_.buffer_slices[self.orientation]
+            buffer_.discard_image()
+            if vtk_cache:
+                buffer_.discard_vtk_image()
+
+    def discard_mask_cache(self, all_orientations=False, vtk_cache=True):
+        if all_orientations:
+            for orientation in self.slice_.buffer_slices:
+                buffer_ = self.slice_.buffer_slices[orientation]
+                buffer_.discard_mask()
+                if vtk_cache:
+                    buffer_.discard_vtk_mask()
+
+        else:
+            buffer_ = self.slice_.buffer_slices[self.orientation]
+            buffer_.discard_mask()
+            if vtk_cache:
+                buffer_.discard_vtk_mask()
