@@ -19,11 +19,12 @@
 import plistlib
 import os
 import weakref
+from distutils.version import LooseVersion
 
 import numpy
 import vtk
 import wx
-from wx.lib.pubsub import pub as Publisher
+from invesalius.pubsub import pub as Publisher
 
 import invesalius.constants as const
 import invesalius.project as prj
@@ -353,9 +354,11 @@ class Volume():
         color_transfer.RemoveAllPoints()
         color_preset = self.config['CLUT']
         if color_preset != "No CLUT":
-            p = plistlib.readPlist(
-                os.path.join(inv_paths.RAYCASTING_PRESETS_DIRECTORY,
-                             'color_list', color_preset + '.plist'))
+            path = os.path.join(inv_paths.RAYCASTING_PRESETS_DIRECTORY,
+                                'color_list', color_preset + '.plist')
+            with open(path, 'rb') as f:
+                p = plistlib.load(f, fmt=plistlib.FMT_XML)
+
             r = p['Red']
             g = p['Green']
             b = p['Blue']
@@ -612,6 +615,7 @@ class Volume():
                 volume_mapper.IntermixIntersectingGeometryOn()
             else:
                 volume_mapper = vtk.vtkGPUVolumeRayCastMapper()
+                volume_mapper.UseJitteringOn()
                 self.volume_mapper = volume_mapper
 
         self.SetTypeRaycasting()
@@ -636,14 +640,15 @@ class Volume():
         except NameError:
             pass
 
-        # Using these lines to improve the raycasting quality. These values
-        # seems related to the distance from ray from raycasting.
-        # TODO: Need to see values that improve the quality and don't decrease
-        # the performance. 2.0 seems to be a good value to pix_diag
-        pix_diag = 2.0
-        volume_mapper.SetImageSampleDistance(0.25)
-        volume_mapper.SetSampleDistance(pix_diag / 5.0)
-        volume_properties.SetScalarOpacityUnitDistance(pix_diag)
+        if not self.volume_mapper.IsA("vtkGPUVolumeRayCastMapper"):
+            # Using these lines to improve the raycasting quality. These values
+            # seems related to the distance from ray from raycasting.
+            # TODO: Need to see values that improve the quality and don't decrease
+            # the performance. 2.0 seems to be a good value to pix_diag
+            pix_diag = 2.0
+            volume_mapper.SetImageSampleDistance(0.25)
+            volume_mapper.SetSampleDistance(pix_diag / 5.0)
+            volume_properties.SetScalarOpacityUnitDistance(pix_diag)
 
         self.volume_properties = volume_properties
 
@@ -705,9 +710,93 @@ class Volume():
         #else:
         #    valor = value 
         return value - scale[0]
-   
-        
-class CutPlane: 
+
+class VolumeMask:
+    def __init__(self, mask):
+        self.mask = mask
+        self.colour = mask.colour
+        self._volume_mapper = None
+        self._flip = None
+        self._color_transfer = None
+        self._piecewise_function = None
+        self._actor = None
+
+    def create_volume(self):
+        if self._actor is None:
+            if int(ses.Session().rendering) == 0:
+                self._volume_mapper = vtk.vtkFixedPointVolumeRayCastMapper()
+                #volume_mapper.AutoAdjustSampleDistancesOff()
+                self._volume_mapper.IntermixIntersectingGeometryOn()
+                pix_diag = 2.0
+                self._volume_mapper.SetImageSampleDistance(0.25)
+                self._volume_mapper.SetSampleDistance(pix_diag / 5.0)
+            else:
+                self._volume_mapper = vtk.vtkGPUVolumeRayCastMapper()
+                self._volume_mapper.UseJitteringOn()
+
+                if LooseVersion(vtk.vtkVersion().GetVTKVersion()) > LooseVersion('8.0'):
+                    self._volume_mapper.SetBlendModeToIsoSurface()
+
+            #  else:
+                #  isosurfaceFunc = vtk.vtkVolumeRayCastIsosurfaceFunction()
+                #  isosurfaceFunc.SetIsoValue(127)
+
+                #  self._volume_mapper = vtk.vtkVolumeRayCastMapper()
+                #  self._volume_mapper.SetVolumeRayCastFunction(isosurfaceFunc)
+
+            self._flip = vtk.vtkImageFlip()
+            self._flip.SetInputData(self.mask.imagedata)
+            self._flip.SetFilteredAxis(1)
+            self._flip.FlipAboutOriginOn()
+
+            self._volume_mapper.SetInputConnection(self._flip.GetOutputPort())
+            self._volume_mapper.Update()
+
+            r, g, b = self.colour
+
+            self._color_transfer = vtk.vtkColorTransferFunction()
+            self._color_transfer.RemoveAllPoints()
+            self._color_transfer.AddRGBPoint(0.0, 0, 0, 0)
+            self._color_transfer.AddRGBPoint(254.0, r, g, b)
+            self._color_transfer.AddRGBPoint(255.0, r, g, b)
+
+            self._piecewise_function = vtk.vtkPiecewiseFunction()
+            self._piecewise_function.RemoveAllPoints()
+            self._piecewise_function.AddPoint(0.0, 0.0)
+            self._piecewise_function.AddPoint(127, 1.0)
+
+            self._volume_property = vtk.vtkVolumeProperty()
+            self._volume_property.SetColor(self._color_transfer)
+            self._volume_property.SetScalarOpacity(self._piecewise_function)
+            self._volume_property.ShadeOn()
+            self._volume_property.SetInterpolationTypeToLinear()
+            self._volume_property.SetSpecular(0.75)
+            self._volume_property.SetSpecularPower(2)
+
+            if not self._volume_mapper.IsA("vtkGPUVolumeRayCastMapper"):
+                self._volume_property.SetScalarOpacityUnitDistance(pix_diag)
+            else:
+                if LooseVersion(vtk.vtkVersion().GetVTKVersion()) > LooseVersion('8.0'):
+                    self._volume_property.GetIsoSurfaceValues().SetValue(0, 127)
+
+            self._actor = vtk.vtkVolume()
+            self._actor.SetMapper(self._volume_mapper)
+            self._actor.SetProperty(self._volume_property)
+            self._actor.Update()
+
+    def change_imagedata(self):
+        self._flip.SetInputData(self.mask.imagedata)
+
+    def set_colour(self, colour):
+        self.colour = colour
+        r, g, b = self.colour
+        self._color_transfer.RemoveAllPoints()
+        self._color_transfer.AddRGBPoint(0.0, 0, 0, 0)
+        self._color_transfer.AddRGBPoint(254.0, r, g, b)
+        self._color_transfer.AddRGBPoint(255.0, r, g, b)
+
+
+class CutPlane:
     def __init__(self, img, volume_mapper):
         self.img = img
         self.volume_mapper = volume_mapper
