@@ -20,7 +20,7 @@ import sys
 
 import vtk
 import wx
-from wx.lib.pubsub import pub as Publisher
+from invesalius.pubsub import pub as Publisher
 import invesalius.constants as const
 from invesalius.gui.dialogs import ProgressDialog
 
@@ -45,8 +45,8 @@ def ShowProgress(number_of_filters = 1,
     last_obj_progress = [0]
     if (dialog_type == "ProgressDialog"):
         try:
-            dlg = ProgressDialog(100)
-        except wx._core.PyNoAppError:
+            dlg = ProgressDialog(wx.GetApp().GetTopWindow(), 100)
+        except (wx._core.PyNoAppError, AttributeError):
             return lambda obj, label: 0
 
 
@@ -148,7 +148,11 @@ class Text(object):
             except(UnicodeEncodeError):
                 self.mapper.SetInput(value.encode("utf-8", errors='replace'))
 
+    def GetValue(self):
+        return self.mapper.GetInput()
+
     def SetCoilDistanceValue(self, value):
+        #TODO: Not being used anymore. Can be deleted.
         if isinstance(value, int) or isinstance(value, float):
             value = 'Dist: ' + str("{:06.2f}".format(value)) + ' mm'
             if sys.platform == 'win32':
@@ -218,6 +222,8 @@ class TextZero(object):
         self.text = ''
         self.position = (0, 0)
         self.symbolic_syze = wx.FONTSIZE_MEDIUM
+        self.bottom_pos = False
+        self.right_pos = False
 
     def SetColour(self, colour):
         self.property.SetColor(colour)
@@ -227,6 +233,7 @@ class TextZero(object):
 
     def SetSize(self, size):
         self.property.SetFontSize(size)
+        self.actor.GetTextProperty().ShallowCopy(self.property)
 
     def SetSymbolicSize(self, size):
         self.symbolic_syze = size
@@ -281,8 +288,31 @@ class TextZero(object):
         coord.SetCoordinateSystemToNormalizedDisplay()
         coord.SetValue(*self.position)
         x, y = coord.GetComputedDisplayValue(canvas.evt_renderer)
-
         font = wx.SystemSettings.GetFont(wx.SYS_DEFAULT_GUI_FONT)
-        #  font.SetWeight(wx.FONTWEIGHT_BOLD)
         font.SetSymbolicSize(self.symbolic_syze)
+        font.Scale(canvas.viewer.GetContentScaleFactor())
+        if self.bottom_pos or self.right_pos:
+            w, h = canvas.calc_text_size(self.text, font)
+            if self.right_pos:
+                x -= w
+            if self.bottom_pos:
+                y += h
         canvas.draw_text(self.text, (x, y), font=font)
+
+
+def numpy_to_vtkMatrix4x4(affine):
+    """
+    Convert a numpy 4x4 array to a vtk 4x4 matrix
+    :param affine: 4x4 array
+    :return: vtkMatrix4x4 object representing the affine
+    """
+    # test for type and shape of affine matrix
+    # assert isinstance(affine, np.ndarray)
+    assert affine.shape == (4, 4)
+
+    affine_vtk = vtk.vtkMatrix4x4()
+    for row in range(0, 4):
+        for col in range(0, 4):
+            affine_vtk.SetElement(row, col, affine[row, col])
+
+    return affine_vtk
