@@ -20,8 +20,10 @@
 import math
 import os.path
 import platform
+import subprocess
 import sys
 import webbrowser
+import errno
 
 import invesalius.constants as const
 import invesalius.gui.default_tasks as tasks
@@ -39,8 +41,9 @@ import wx.aui
 import wx.lib.agw.toasterbox as TB
 import wx.lib.popupctl as pc
 from invesalius import inv_paths
+from invesalius.gui import project_properties
 from wx.lib.agw.aui.auibar import AUI_TB_PLAIN_BACKGROUND, AuiToolBar
-from wx.lib.pubsub import pub as Publisher
+from invesalius.pubsub import pub as Publisher
 
 try:
     from wx.adv import TaskBarIcon as wx_TaskBarIcon
@@ -117,6 +120,7 @@ class Frame(wx.Frame):
         self.actived_interpolated_slices = main_menu.view_menu
         self.actived_navigation_mode = main_menu.mode_menu
         self.actived_dbs_mode = main_menu.mode_dbs
+        self.tools_menu = main_menu.tools_menu
 
         # Set menus, status and task bar
         self.SetMenuBar(main_menu)
@@ -128,7 +132,6 @@ class Frame(wx.Frame):
         # Create aui manager and insert content in it
         self.__init_aui()
 
-        self.preferences = preferences.Preferences(self)
         # Initialize bind to pubsub events
         self.__bind_events()
         self.__bind_events_wx()
@@ -214,7 +217,7 @@ class Frame(wx.Frame):
         # Add toolbars to manager
         # This is pretty tricky -- order on win32 is inverted when
         # compared to linux2 & darwin
-        if sys.platform == 'win32':
+        if sys.platform == 'win32' or wx.VERSION >= (4, 1):
             t1 = ProjectToolBar(self)
             t2 = HistoryToolBar(self)
             t3 = LayoutToolBar(self)
@@ -259,6 +262,8 @@ class Frame(wx.Frame):
         # TODO: Allow saving and restoring perspectives
         self.perspective_all = aui_manager.SavePerspective()
 
+        self.Layout()
+
     def _BeginBusyCursor(self):
         """
         Start busy cursor.
@@ -435,6 +440,8 @@ class Frame(wx.Frame):
             self.ShowSaveAsProject()
         elif id == const.ID_EXPORT_SLICE:
             self.ExportProject()
+        elif id == const.ID_PROJECT_PROPERTIES:
+            self.ShowProjectProperties()
         elif id == const.ID_PROJECT_CLOSE:
             self.CloseProject()
         elif id == const.ID_EXIT:
@@ -478,6 +485,10 @@ class Frame(wx.Frame):
             ddlg = dlg.MaskDensityDialog(self)
             ddlg.Show()
 
+        elif id == const.ID_MANUAL_WWWL:
+            wwwl_dlg = dlg.ManualWWWLDialog(self)
+            wwwl_dlg.Show()
+
         elif id == const.ID_THRESHOLD_SEGMENTATION:
             Publisher.sendMessage("Show panel", panel_id=const.ID_THRESHOLD_SEGMENTATION)
             Publisher.sendMessage('Disable actual style')
@@ -508,6 +519,9 @@ class Frame(wx.Frame):
         elif id == const.ID_FLOODFILL_SEGMENTATION:
             self.OnFFillSegmentation()
 
+        elif id == const.ID_SEGMENTATION_BRAIN:
+            self.OnBrainSegmentation()
+
         elif id == const.ID_VIEW_INTERPOLATED:
             st = self.actived_interpolated_slices.IsChecked(const.ID_VIEW_INTERPOLATED)
             if st:
@@ -529,12 +543,24 @@ class Frame(wx.Frame):
         elif id == const.ID_CROP_MASK:
             self.OnCropMask()
 
+        elif id == const.ID_MASK_3D_PREVIEW:
+            self.OnEnableMask3DPreview(value=self.tools_menu.IsChecked(const.ID_MASK_3D_PREVIEW))
+
+        elif id == const.ID_MASK_3D_AUTO_RELOAD:
+            ses.Session().auto_reload_preview = self.tools_menu.IsChecked(const.ID_MASK_3D_AUTO_RELOAD)
+
+        elif id == const.ID_MASK_3D_RELOAD:
+            self.OnUpdateMaskPreview()
+
         elif id == const.ID_CREATE_SURFACE:
             Publisher.sendMessage('Open create surface dialog')
 
         elif id == const.ID_CREATE_MASK:
             Publisher.sendMessage('New mask from shortcut')
 
+        elif id == const.ID_PLUGINS_SHOW_PATH:
+            self.ShowPluginsFolder()
+
     def OnDbsMode(self):
         st = self.actived_dbs_mode.IsChecked()
         Publisher.sendMessage('Deactive target button')
@@ -580,12 +606,13 @@ class Frame(wx.Frame):
         self.mw.SetPosition(pos)
 
     def ShowPreferences(self):
+        preferences_dialog = preferences.Preferences(self)
+        preferences_dialog.LoadPreferences()
+        preferences_dialog.Center()
 
-        self.preferences.Center()
-        
-        if self.preferences.ShowModal() == wx.ID_OK:
-            values = self.preferences.GetPreferences()
-            self.preferences.Close()
+        if preferences_dialog.ShowModal() == wx.ID_OK:
+            values = preferences_dialog.GetPreferences()
+            preferences_dialog.Destroy()
 
             ses.Session().rendering = values[const.RENDERING]
             ses.Session().surface_interpolation = values[const.SURFACE_INTERPOLATION]
@@ -661,19 +688,44 @@ class Frame(wx.Frame):
 
         session = ses.Session()
         last_directory = session.get('paths', 'last_directory_export_prj', '')
-        dlg = wx.FileDialog(None,
+        fdlg = wx.FileDialog(None,
                             "Export slice ...",
                             last_directory, # last used directory
                             os.path.split(p.name)[-1], # initial filename
                             WILDCARD_EXPORT_SLICE,
                             wx.FD_SAVE|wx.FD_OVERWRITE_PROMPT)
-        if dlg.ShowModal() == wx.ID_OK:
-            filename = dlg.GetPath()
-            ext = IDX_EXT[dlg.GetFilterIndex()]
+        if fdlg.ShowModal() == wx.ID_OK:
+            filename = fdlg.GetPath()
+            ext = IDX_EXT[fdlg.GetFilterIndex()]
+            dirpath = os.path.split(filename)[0]
             if not filename.endswith(ext):
                 filename += ext
-            p.export_project(filename)
-            session['paths']['last_directory_export_prj'] = os.path.split(filename)[0]
+            try:
+                p.export_project(filename)
+            except (OSError, IOError) as err:
+                if err.errno == errno.EACCES:
+                    message = "It was not possible to save because you don't have permission to write at {}".format(dirpath)
+                else:
+                    message = "It was not possible to save because"
+                d = dlg.ErrorMessageBox(
+                    None,
+                    "Save project error",
+                    "{}:\n{}".format(message, err)
+                )
+                d.ShowModal()
+                d.Destroy()
+            else:
+                session['paths']['last_directory_export_prj'] = dirpath
+
+    def ShowProjectProperties(self):
+        window = project_properties.ProjectProperties(self)
+        if window.ShowModal() == wx.ID_OK:
+            p = prj.Project()
+            if window.name_txt.GetValue() != p.name:
+                p.name = window.name_txt.GetValue()
+                ses.Session().ChangeProject()
+                self._SetProjectName(p.name)
+        window.Destroy()
 
     def ShowBitmapImporter(self):
         """
@@ -736,12 +788,48 @@ class Frame(wx.Frame):
     def OnFFillSegmentation(self):
         Publisher.sendMessage('Enable style', style=const.SLICE_STATE_FFILL_SEGMENTATION)
 
+    def OnBrainSegmentation(self):
+        from invesalius.gui import brain_seg_dialog
+        if brain_seg_dialog.HAS_PLAIDML or brain_seg_dialog.HAS_THEANO or brain_seg_dialog.HAS_TORCH:
+            dlg = brain_seg_dialog.BrainSegmenterDialog(self)
+            dlg.Show()
+        else:
+            dlg = wx.MessageDialog(self,
+                                   _("It's not possible to run brain segmenter because your system doesn't have the following modules installed:") \
+                                   + " Torch, PlaidML or Theano" ,
+                                   "InVesalius 3 - Brain segmenter",
+                                   wx.ICON_INFORMATION | wx.OK)
+            dlg.ShowModal()
+            dlg.Destroy()
+
     def OnInterpolatedSlices(self, status):
         Publisher.sendMessage('Set interpolated slices', flag=status)
 
     def OnCropMask(self):
         Publisher.sendMessage('Enable style', style=const.SLICE_STATE_CROP_MASK)
 
+    def OnEnableMask3DPreview(self, value):
+        if value:
+            Publisher.sendMessage('Enable mask 3D preview')
+        else:
+            Publisher.sendMessage('Disable mask 3D preview')
+
+    def OnUpdateMaskPreview(self):
+        Publisher.sendMessage('Update mask 3D preview')
+
+    def ShowPluginsFolder(self):
+        """
+        Show getting started window.
+        """
+        inv_paths.create_conf_folders()
+        path = str(inv_paths.USER_PLUGINS_DIRECTORY)
+        if platform.system() == "Windows":
+            os.startfile(path)
+        elif platform.system() == "Darwin":
+            subprocess.Popen(["open", path])
+        else:
+            subprocess.Popen(["xdg-open", path])
+
 # ------------------------------------------------------------------
 # ------------------------------------------------------------------
 # ------------------------------------------------------------------
@@ -755,12 +843,14 @@ class MenuBar(wx.MenuBar):
         wx.MenuBar.__init__(self)
 
         self.parent = parent
+        self._plugins_menu_ids = {}
 
         # Used to enable/disable menu items if project is opened or
         # not. Eg. save should only be available if a project is open
         self.enable_items = [const.ID_PROJECT_SAVE,
                              const.ID_PROJECT_SAVE_AS,
                              const.ID_EXPORT_SLICE,
+                             const.ID_PROJECT_PROPERTIES,
                              const.ID_PROJECT_CLOSE,
                              const.ID_REORIENT_IMG,
                              const.ID_FLOODFILL_MASK,
@@ -779,10 +869,12 @@ class MenuBar(wx.MenuBar):
                              const.ID_WATERSHED_SEGMENTATION,
                              const.ID_THRESHOLD_SEGMENTATION,
                              const.ID_FLOODFILL_SEGMENTATION,
+                             const.ID_SEGMENTATION_BRAIN,
                              const.ID_MASK_DENSITY_MEASURE,
                              const.ID_CREATE_SURFACE,
                              const.ID_CREATE_MASK,
-                             const.ID_GOTO_SLICE]
+                             const.ID_GOTO_SLICE,
+                             const.ID_MANUAL_WWWL]
         self.__init_items()
         self.__bind_events()
 
@@ -800,7 +892,7 @@ class MenuBar(wx.MenuBar):
         sub(self.OnEnableState, "Enable state project")
         sub(self.OnEnableUndo, "Enable undo")
         sub(self.OnEnableRedo, "Enable redo")
-        sub(self.OnEnableGotoCoord, "Update affine matrix")
+        sub(self.OnEnableGotoCoord, "Enable Go-to-Coord")
         sub(self.OnEnableNavigation, "Navigation status")
 
         sub(self.OnAddMask, "Add mask")
@@ -809,6 +901,8 @@ class MenuBar(wx.MenuBar):
         sub(self.OnUpdateSliceInterpolation, "Update Slice Interpolation MenuBar")
         sub(self.OnUpdateNavigationMode, "Update Navigation Mode MenuBar")
 
+        sub(self.AddPluginsItems, "Add plugins menu items")
+
         self.num_masks = 0
 
     def __init_items(self):
@@ -834,6 +928,7 @@ class MenuBar(wx.MenuBar):
         app(const.ID_PROJECT_SAVE, _("Save project\tCtrl+S"))
         app(const.ID_PROJECT_SAVE_AS, _("Save project as...\tCtrl+Shift+S"))
         app(const.ID_EXPORT_SLICE, _("Export project"))
+        app(const.ID_PROJECT_PROPERTIES, _("Project properties"))
         app(const.ID_PROJECT_CLOSE, _("Close project"))
         file_menu.AppendSeparator()
         #app(const.ID_PROJECT_INFO, _("Project Information..."))
@@ -909,6 +1004,22 @@ class MenuBar(wx.MenuBar):
         self.crop_mask_menu = mask_menu.Append(const.ID_CROP_MASK, _("Crop"))
         self.crop_mask_menu.Enable(False)
 
+        mask_menu.AppendSeparator()
+
+        mask_preview_menu = wx.Menu()
+
+        self.mask_preview = mask_preview_menu.Append(const.ID_MASK_3D_PREVIEW, _("Enable") + "\tCtrl+Shift+P", "", wx.ITEM_CHECK)
+        self.mask_preview.Enable(False)
+
+        self.mask_auto_reload = mask_preview_menu.Append(const.ID_MASK_3D_AUTO_RELOAD, _("Auto reload") + "\tCtrl+Shift+D", "", wx.ITEM_CHECK)
+        self.mask_auto_reload.Check(ses.Session().auto_reload_preview)
+        self.mask_auto_reload.Enable(False)
+
+        self.mask_preview_reload = mask_preview_menu.Append(const.ID_MASK_3D_RELOAD, _("Reload") + "\tCtrl+Shift+R")
+        self.mask_preview_reload.Enable(False)
+
+        mask_menu.Append(-1, _('Mask 3D Preview'), mask_preview_menu)
+
         # Segmentation Menu
         segmentation_menu = wx.Menu()
         self.threshold_segmentation = segmentation_menu.Append(const.ID_THRESHOLD_SEGMENTATION, _(u"Threshold\tCtrl+Shift+T"))
@@ -916,6 +1027,8 @@ class MenuBar(wx.MenuBar):
         self.watershed_segmentation = segmentation_menu.Append(const.ID_WATERSHED_SEGMENTATION, _(u"Watershed\tCtrl+Shift+W"))
         self.ffill_segmentation = segmentation_menu.Append(const.ID_FLOODFILL_SEGMENTATION, _(u"Region growing\tCtrl+Shift+G"))
         self.ffill_segmentation.Enable(False)
+        segmentation_menu.AppendSeparator()
+        segmentation_menu.Append(const.ID_SEGMENTATION_BRAIN, _("Brain segmentation (MRI T1)"))
 
         # Surface Menu
         surface_menu = wx.Menu()
@@ -941,12 +1054,14 @@ class MenuBar(wx.MenuBar):
 
         mask_density_menu = image_menu.Append(const.ID_MASK_DENSITY_MEASURE, _(u'Mask Density measure'))
         reorient_menu = image_menu.Append(const.ID_REORIENT_IMG, _(u'Reorient image\tCtrl+Shift+R'))
+        image_menu.Append(const.ID_MANUAL_WWWL, _("Set WW&&WL manually"))
 
         reorient_menu.Enable(False)
         tools_menu.Append(-1, _(u'Image'), image_menu)
         tools_menu.Append(-1,  _(u"Mask"), mask_menu)
         tools_menu.Append(-1, _(u"Segmentation"), segmentation_menu)
         tools_menu.Append(-1, _(u"Surface"), surface_menu)
+        self.tools_menu = tools_menu
 
         #View
         self.view_menu = view_menu = wx.Menu()
@@ -1003,6 +1118,10 @@ class MenuBar(wx.MenuBar):
 
         self.actived_navigation_mode = self.mode_menu
 
+        plugins_menu = wx.Menu()
+        plugins_menu.Append(const.ID_PLUGINS_SHOW_PATH, _("Open Plugins folder"))
+        self.plugins_menu = plugins_menu
+
         # HELP
         help_menu = wx.Menu()
         help_menu.Append(const.ID_START, _("Getting started..."))
@@ -1020,11 +1139,24 @@ class MenuBar(wx.MenuBar):
         self.Append(file_edit, _("Edit"))
         self.Append(view_menu, _(u"View"))
         self.Append(tools_menu, _(u"Tools"))
+        self.Append(plugins_menu, _(u"Plugins"))
         #self.Append(tools_menu, "Tools")
         self.Append(options_menu, _("Options"))
         self.Append(mode_menu, _("Mode"))
         self.Append(help_menu, _("Help"))
 
+        plugins_menu.Bind(wx.EVT_MENU, self.OnPluginMenu)
+
+    def OnPluginMenu(self, evt):
+        id = evt.GetId()
+        if id != const.ID_PLUGINS_SHOW_PATH:
+            try:
+                plugin_name = self._plugins_menu_ids[id]["name"]
+                print("Loading plugin:", plugin_name)
+                Publisher.sendMessage("Load plugin", plugin_name=plugin_name)
+            except KeyError:
+                print("Invalid plugin")
+        evt.Skip()
 
     def SliceInterpolationStatus(self):
         
@@ -1052,6 +1184,18 @@ class MenuBar(wx.MenuBar):
         v = self.NavigationModeStatus()
         self.mode_menu.Check(const.ID_MODE_NAVIGATION, v)
 
+    def AddPluginsItems(self, items):
+        for menu_item in self.plugins_menu.GetMenuItems():
+            if menu_item.GetId() != const.ID_PLUGINS_SHOW_PATH:
+                self.plugins_menu.DestroyItem(menu_item)
+
+        for item in items:
+            _new_id = wx.NewId()
+            self._plugins_menu_ids[_new_id] = items[item]
+            menu_item = self.plugins_menu.Append(_new_id, item, items[item]["description"])
+            menu_item.Enable(items[item]["enable_startup"])
+            print(">>> menu", item)
+
     def OnEnableState(self, state):
         """
         Based on given state, enables or disables menu items which
@@ -1069,6 +1213,11 @@ class MenuBar(wx.MenuBar):
         for item in self.enable_items:
             self.Enable(item, False)
 
+        # Disabling plugins menus that needs a project open
+        for item in self._plugins_menu_ids:
+            if not self._plugins_menu_ids[item]["enable_startup"]:
+                self.Enable(item, False)
+
     def SetStateProjectOpen(self):
         """
         Enable menu items (e.g. save) when project is opened.
@@ -1076,6 +1225,11 @@ class MenuBar(wx.MenuBar):
         for item in self.enable_items:
             self.Enable(item, True)
 
+        # Enabling plugins menus that needs a project open
+        for item in self._plugins_menu_ids:
+            if not self._plugins_menu_ids[item]["enable_startup"]:
+                self.Enable(item, True)
+
     def OnEnableUndo(self, value):
         if value:
             self.FindItemById(wx.ID_UNDO).Enable(True)
@@ -1088,22 +1242,21 @@ class MenuBar(wx.MenuBar):
         else:
             self.FindItemById(wx.ID_REDO).Enable(False)
 
-    def OnEnableGotoCoord(self,  affine, status):
+    def OnEnableGotoCoord(self, status=True):
         """
-        Disable goto coord either if there is no affine matrix or affine is wrongly imported.
-        :param status: Affine matrix status
+        Enable or disable goto coord depending on the imported affine matrix.
+        :param status: True for enabling and False for disabling the Go-To-Coord
         """
-        if status:
-            self.FindItemById(const.ID_GOTO_COORD).Enable(True)
-        else:
-            self.FindItemById(const.ID_GOTO_COORD).Enable(False)
 
-    def OnEnableNavigation(self, status):
+        self.FindItemById(const.ID_GOTO_COORD).Enable(status)
+
+    def OnEnableNavigation(self, nav_status, vis_status):
         """
         Disable mode menu when navigation is on.
-        :param status: Navigation status
+        :param nav_status: Navigation status
+        :param vis_status: Status of items visualization during navigation
         """
-        value = status
+        value = nav_status
         if value:
             self.FindItemById(const.ID_MODE_NAVIGATION).Enable(False)
         else:
@@ -1112,6 +1265,9 @@ class MenuBar(wx.MenuBar):
     def OnAddMask(self, mask):
         self.num_masks += 1
         self.bool_op_menu.Enable(self.num_masks >= 2)
+        self.mask_preview.Enable(True)
+        self.mask_auto_reload.Enable(True)
+        self.mask_preview_reload.Enable(True)
 
     def OnRemoveMasks(self, mask_indexes):
         self.num_masks -= len(mask_indexes)
@@ -1120,6 +1276,9 @@ class MenuBar(wx.MenuBar):
     def OnShowMask(self, index, value):
         self.clean_mask_menu.Enable(value)
         self.crop_mask_menu.Enable(value)
+        self.mask_preview.Enable(value)
+        self.mask_auto_reload.Enable(value)
+        self.mask_preview_reload.Enable(value)
 
 
 # ------------------------------------------------------------------
@@ -1856,11 +2015,11 @@ class LayoutToolBar(AuiToolBar):
         d = inv_paths.ICON_DIR
         if sys.platform == 'darwin':
             # Bitmaps for show/hide task panel item
-            p = os.path.join(d, "layout_data_only_original.gif")
-            self.BMP_WITH_MENU = wx.Bitmap(str(p), wx.BITMAP_TYPE_GIF)
+            p = os.path.join(d, "layout_data_only_original.png")
+            self.BMP_WITH_MENU = wx.Bitmap(str(p), wx.BITMAP_TYPE_PNG)
 
-            p = os.path.join(d, "layout_full_original.gif")
-            self.BMP_WITHOUT_MENU = wx.Bitmap(str(p), wx.BITMAP_TYPE_GIF)
+            p = os.path.join(d, "layout_full_original.png")
+            self.BMP_WITHOUT_MENU = wx.Bitmap(str(p), wx.BITMAP_TYPE_PNG)
 
             # Bitmaps for show/hide task item
             p = os.path.join(d, "text_inverted_original.png")
@@ -1871,11 +2030,11 @@ class LayoutToolBar(AuiToolBar):
 
         else:
             # Bitmaps for show/hide task panel item
-            p = os.path.join(d, "layout_data_only.gif")
-            self.BMP_WITH_MENU = wx.Bitmap(str(p), wx.BITMAP_TYPE_GIF)
+            p = os.path.join(d, "layout_data_only.png")
+            self.BMP_WITH_MENU = wx.Bitmap(str(p), wx.BITMAP_TYPE_PNG)
 
-            p = os.path.join(d, "layout_full.gif")
-            self.BMP_WITHOUT_MENU = wx.Bitmap(str(p), wx.BITMAP_TYPE_GIF)
+            p = os.path.join(d, "layout_full.png")
+            self.BMP_WITHOUT_MENU = wx.Bitmap(str(p), wx.BITMAP_TYPE_PNG)
 
             # Bitmaps for show/hide task item
             p = os.path.join(d, "text_inverted.png")
