@@ -18,6 +18,7 @@
 #--------------------------------------------------------------------------
 
 import os
+import pathlib
 import sys
 
 import wx
@@ -28,7 +29,7 @@ except ImportError:
     import wx.lib.hyperlink as hl
 
 import wx.lib.platebtn as pbtn
-from wx.lib.pubsub import pub as Publisher
+from invesalius.pubsub import pub as Publisher
 
 import invesalius.constants as const
 import invesalius.gui.dialogs as dlg
@@ -317,9 +318,9 @@ class InnerTaskPanel(wx.Panel):
 
         if n_surface:
             if sys.platform == 'win32':
-                project_name = project.name
+                project_name = pathlib.Path(project.name).stem
             else:
-                project_name = project.name+".stl"
+                project_name = pathlib.Path(project.name).stem + ".stl"
 
             session = ses.Session()
             last_directory = session.get('paths', 'last_directory_3d_surface', '')
@@ -347,13 +348,6 @@ class InnerTaskPanel(wx.Panel):
 
                 Publisher.sendMessage('Export surface to file',
                                       filename=filename, filetype=filetype)
-                if not os.path.exists(filename):
-                    dlg = wx.MessageDialog(None,
-                                       _("It was not possible to save the surface."),
-                                       _("Error saving surface"),
-                                       wx.OK | wx.ICON_ERROR)
-                    dlg.ShowModal()
-                    dlg.Destroy()
         else:
             dlg = wx.MessageDialog(None,
                     _("You need to create a surface and make it ") +
