@@ -31,7 +31,7 @@ except ImportError:
 
 import wx.lib.platebtn as pbtn
 import wx.lib.colourselect as csel
-from wx.lib.pubsub import pub as Publisher
+from invesalius.pubsub import pub as Publisher
 
 import invesalius.data.mask as mask
 import invesalius.data.slice_ as slice_
@@ -141,7 +141,7 @@ class InnerTaskPanel(wx.Panel):
 
         line_sizer = wx.BoxSizer(wx.HORIZONTAL)
         line_sizer.Add(check_box, 0, wx.ALIGN_LEFT|wx.RIGHT|wx.LEFT, 5)
-        line_sizer.Add(next_btn_sizer, 1, wx.EXPAND|wx.ALIGN_RIGHT|wx.RIGHT|wx.LEFT, 5)
+        line_sizer.Add(next_btn_sizer, 1, wx.EXPAND|wx.RIGHT|wx.LEFT, 5)
         line_sizer.Fit(self)
 
         # Add line sizers into main sizer
@@ -182,7 +182,14 @@ class InnerTaskPanel(wx.Panel):
                 dlgs.Destroy()
 
             if to_generate:
-                mask_index = sl.current_mask.index
+                proj = Project()
+                for idx in proj.mask_dict:
+                    if proj.mask_dict[idx] is sl.current_mask:
+                        mask_index = idx
+                        break
+                else:
+                    return
+
                 method = {'algorithm': algorithm,
                           'options': options}
                 srf_options = {"index": mask_index,
@@ -423,6 +430,7 @@ class InnerFoldPanel(wx.Panel):
         try:
             _id = panel_seg_id[panel_id]
             self.fold_panel.Expand(self.fold_panel.GetFoldPanel(_id))
+            self.Layout()
         except KeyError:
             pass
 
@@ -564,7 +572,7 @@ class MaskProperties(wx.Panel):
         self.bind_evt_gradient = True
         thresh = (thresh_min, thresh_max)
         if thresh in Project().threshold_modes.values():
-            preset_name = Project().threshold_modes.get_key(thresh)[0]
+            preset_name = Project().threshold_modes.get_key(thresh)
             index = self.threshold_modes_names.index(preset_name)
             self.combo_thresh.SetSelection(index)
         else:
@@ -578,7 +586,7 @@ class MaskProperties(wx.Panel):
         self.gradient.SetMaxValue(thresh_max)
         thresh = (thresh_min, thresh_max)
         if thresh in Project().threshold_modes.values():
-            preset_name = Project().threshold_modes.get_key(thresh)[0]
+            preset_name = Project().threshold_modes.get_key(thresh)
             index = self.threshold_modes_names.index(preset_name)
             self.combo_thresh.SetSelection(index)
         else:
@@ -620,7 +628,7 @@ class MaskProperties(wx.Panel):
             thresh_min, thresh_max = self.threshold_modes[default_thresh]
 
         elif default_thresh in proj.threshold_modes.values():
-            preset_name = proj.threshold_modes.get_key(default_thresh)[0]
+            preset_name = proj.threshold_modes.get_key(default_thresh)
             index = self.threshold_modes_names.index(preset_name)
             self.combo_thresh.SetSelection(index)
             thresh_min, thresh_max = default_thresh
@@ -688,11 +696,11 @@ class EditionTools(wx.Panel):
         ## LINE 2
         menu = wx.Menu()
 
-        CIRCLE_BMP = wx.Bitmap(os.path.join(inv_paths.ICON_DIR, "brush_circle.jpg"), wx.BITMAP_TYPE_JPEG)
+        CIRCLE_BMP = wx.Bitmap(os.path.join(inv_paths.ICON_DIR, "brush_circle.png"), wx.BITMAP_TYPE_PNG)
         item = wx.MenuItem(menu, MENU_BRUSH_CIRCLE, _("Circle"))
         item.SetBitmap(CIRCLE_BMP)
 
-        SQUARE_BMP = wx.Bitmap(os.path.join(inv_paths.ICON_DIR, "brush_square.jpg"), wx.BITMAP_TYPE_JPEG)
+        SQUARE_BMP = wx.Bitmap(os.path.join(inv_paths.ICON_DIR, "brush_square.png"), wx.BITMAP_TYPE_PNG)
         item2 = wx.MenuItem(menu, MENU_BRUSH_SQUARE, _("Square"))
         item2.SetBitmap(SQUARE_BMP)
 
@@ -771,6 +779,7 @@ class EditionTools(wx.Panel):
         Publisher.subscribe(self.SetGradientColour, 'Add mask')
         Publisher.subscribe(self._set_brush_size, 'Set edition brush size')
         Publisher.subscribe(self._set_threshold_range_gui, 'Set edition threshold gui')
+        Publisher.subscribe(self.ChangeMaskColour, 'Set GUI items colour')
 
     def ChangeMaskColour(self, colour):
         self.gradient_thresh.SetColour(colour)
@@ -802,8 +811,8 @@ class EditionTools(wx.Panel):
                                   threshold_range=(thresh_min, thresh_max))
 
     def OnMenu(self, evt):
-        SQUARE_BMP = wx.Bitmap(os.path.join(inv_paths.ICON_DIR, "brush_square.jpg"), wx.BITMAP_TYPE_JPEG)
-        CIRCLE_BMP = wx.Bitmap(os.path.join(inv_paths.ICON_DIR, "brush_circle.jpg"), wx.BITMAP_TYPE_JPEG)
+        SQUARE_BMP = wx.Bitmap(os.path.join(inv_paths.ICON_DIR, "brush_square.png"), wx.BITMAP_TYPE_PNG)
+        CIRCLE_BMP = wx.Bitmap(os.path.join(inv_paths.ICON_DIR, "brush_circle.png"), wx.BITMAP_TYPE_PNG)
 
         brush = {MENU_BRUSH_CIRCLE: const.BRUSH_CIRCLE,
                  MENU_BRUSH_SQUARE: const.BRUSH_SQUARE}
@@ -851,11 +860,11 @@ class WatershedTool(EditionTools):
         ## LINE 2
         menu = wx.Menu()
 
-        CIRCLE_BMP = wx.Bitmap(os.path.join(inv_paths.ICON_DIR, "brush_circle.jpg"), wx.BITMAP_TYPE_JPEG)
+        CIRCLE_BMP = wx.Bitmap(os.path.join(inv_paths.ICON_DIR, "brush_circle.png"), wx.BITMAP_TYPE_PNG)
         item = wx.MenuItem(menu, MENU_BRUSH_CIRCLE, _("Circle"))
         item.SetBitmap(CIRCLE_BMP)
 
-        SQUARE_BMP = wx.Bitmap(os.path.join(inv_paths.ICON_DIR, "brush_square.jpg"), wx.BITMAP_TYPE_JPEG)
+        SQUARE_BMP = wx.Bitmap(os.path.join(inv_paths.ICON_DIR, "brush_square.png"), wx.BITMAP_TYPE_PNG)
         item2 = wx.MenuItem(menu, MENU_BRUSH_SQUARE, _("Square"))
         item2.SetBitmap(SQUARE_BMP)
 
@@ -972,8 +981,8 @@ class WatershedTool(EditionTools):
         self.gradient_thresh.SetMaxValue(thresh_max)
 
     def OnMenu(self, evt):
-        SQUARE_BMP = wx.Bitmap(os.path.join(inv_paths.ICON_DIR, "brush_square.jpg"), wx.BITMAP_TYPE_JPEG)
-        CIRCLE_BMP = wx.Bitmap(os.path.join(inv_paths.ICON_DIR, "brush_circle.jpg"), wx.BITMAP_TYPE_JPEG)
+        SQUARE_BMP = wx.Bitmap(os.path.join(inv_paths.ICON_DIR, "brush_square.png"), wx.BITMAP_TYPE_PNG)
+        CIRCLE_BMP = wx.Bitmap(os.path.join(inv_paths.ICON_DIR, "brush_circle.png"), wx.BITMAP_TYPE_PNG)
 
         brush = {MENU_BRUSH_CIRCLE: const.BRUSH_CIRCLE,
                  MENU_BRUSH_SQUARE: const.BRUSH_SQUARE}
