@@ -30,7 +30,7 @@ except ImportError:
     from weakrefmethod import WeakMethod
 
 from invesalius.data import converters
-from wx.lib.pubsub import pub as Publisher
+from invesalius.pubsub import pub as Publisher
 
 
 class CanvasEvent:
@@ -180,13 +180,11 @@ class CanvasRendererCTX:
         return False
 
     def Refresh(self):
-        print('Refresh')
         self.modified = True
         self.viewer.interactor.Render()
 
     def OnMouseMove(self, evt):
-        x, y = evt.GetPosition()
-        y = self.viewer.interactor.GetSize()[1] - y
+        x, y = self.viewer.get_vtk_mouse_position()
         redraw = False
 
         if self._drag_obj:
@@ -230,8 +228,7 @@ class CanvasRendererCTX:
         evt.Skip()
 
     def OnLeftButtonPress(self, evt):
-        x, y = evt.GetPosition()
-        y = self.viewer.interactor.GetSize()[1] - y
+        x, y = self.viewer.get_vtk_mouse_position()
         if self._over_obj and hasattr(self._over_obj, 'on_mouse_move'):
             if hasattr(self._over_obj, 'on_select'):
                 try:
@@ -286,8 +283,7 @@ class CanvasRendererCTX:
         evt.Skip()
 
     def OnDoubleClick(self, evt):
-        x, y = evt.GetPosition()
-        y = self.viewer.interactor.GetSize()[1] - y
+        x, y = self.viewer.get_vtk_mouse_position()
         evt_obj = CanvasEvent('double_left_click', None, (x, y), self.viewer, self.evt_renderer,
                               control_down=evt.ControlDown(),
                               alt_down=evt.AltDown(),
@@ -301,6 +297,7 @@ class CanvasRendererCTX:
     def OnPaint(self, evt, obj):
         size = self.canvas_renderer.GetSize()
         w, h = size
+        ew, eh = self.evt_renderer.GetSize()
         if self._size != size:
             self._size = size
             self._resize_canvas(w, h)
@@ -586,7 +583,9 @@ class CanvasRendererCTX:
 
         return (xi, yi, xf, yf)
 
-    def draw_rectangle(self, pos, width, height, line_colour=(255, 0, 0, 128), fill_colour=(0, 0, 0, 0)):
+    def draw_rectangle(self, pos, width, height, line_colour=(255, 0, 0, 128),
+        fill_colour=(0, 0, 0, 0), line_width=1, pen_style=wx.PENSTYLE_SOLID,
+        brush_style=wx.BRUSHSTYLE_SOLID):
         """
         Draw a rectangle with its top left at pos and with the given width and height.
 
@@ -603,9 +602,11 @@ class CanvasRendererCTX:
 
         px, py = pos
         py = -py
-        gc.SetPen(wx.Pen(wx.Colour(*line_colour)))
-        gc.SetBrush(wx.Brush(wx.Colour(*fill_colour)))
-        gc.DrawRectangle(px, py, width, height)
+        pen = wx.Pen(wx.Colour(*line_colour), width=line_width, style=pen_style)
+        brush = wx.Brush(wx.Colour(*fill_colour), style=brush_style)
+        gc.SetPen(pen)
+        gc.SetBrush(brush)
+        gc.DrawRectangle(px, py, width, -height)
         self._drawn = True
 
     def draw_text(self, text, pos, font=None, txt_colour=(255, 255, 255)):
@@ -624,17 +625,18 @@ class CanvasRendererCTX:
 
         if font is None:
             font = wx.SystemSettings.GetFont(wx.SYS_DEFAULT_GUI_FONT)
+            font.Scale(self.viewer.GetContentScaleFactor())
 
-        font = gc.CreateFont(font, txt_colour)
-        gc.SetFont(font)
-
+        _font = gc.CreateFont(font, txt_colour)
         px, py = pos
         for t in text.split('\n'):
+            t = t.strip()
             _py = -py
             _px = px
+            gc.SetFont(_font)
             gc.DrawText(t, _px, _py)
 
-            w, h = self.calc_text_size(t)
+            w, h = self.calc_text_size(t, font)
             py -= h
 
         self._drawn = True
@@ -657,16 +659,17 @@ class CanvasRendererCTX:
 
         if font is None:
             font = wx.SystemSettings.GetFont(wx.SYS_DEFAULT_GUI_FONT)
+            font.Scale(self.viewer.GetContentScaleFactor())
 
         _font = gc.CreateFont(font, txt_colour)
         gc.SetFont(_font)
-        w, h = self.calc_text_size(text)
+        w, h = self.calc_text_size(text, font)
 
         px, py = pos
 
         # Drawing the box
         cw, ch = w + border * 2, h + border * 2
-        self.draw_rectangle((px, py), cw, ch, bg_colour, bg_colour)
+        self.draw_rectangle((px, py - ch), cw, ch, bg_colour, bg_colour)
 
         # Drawing the text
         tpx, tpy = px + border, py - border
@@ -817,7 +820,7 @@ class TextBox(CanvasHandlerBase):
                                               bg_colour=self.box_colour)
             if self._highlight:
                 rw, rh = canvas.evt_renderer.GetSize()
-                canvas.draw_rectangle((px, py), w, h,
+                canvas.draw_rectangle((px, py - h), w, h,
                                       (255, 0, 0, 25),
                                       (255, 0, 0, 25))
 
@@ -876,11 +879,13 @@ class CircleHandler(CanvasHandlerBase):
 
     def draw_to_canvas(self, gc, canvas):
         if self.visible:
+            viewer = canvas.viewer
+            scale = viewer.GetContentScaleFactor()
             if self.is_3d:
                 px, py = self._3d_to_2d(canvas.evt_renderer, self.position)
             else:
                 px, py = self.position
-            x, y, w, h = canvas.draw_circle((px, py), self.radius,
+            x, y, w, h = canvas.draw_circle((px, py), self.radius * scale,
                                             line_colour=self.line_colour,
                                             fill_colour=self.fill_colour)
             self.bbox = (x - w/2, y - h/2, x + w/2, y + h/2)
