@@ -26,7 +26,7 @@ except(ImportError):
     from ordereddict import OrderedDict
 
 import wx
-from wx.lib.pubsub import pub as Publisher
+from invesalius.pubsub import pub as Publisher
 
 import invesalius.constants as const
 import invesalius.data.slice_ as sl
@@ -67,7 +67,7 @@ class SliceMenu(wx.Menu):
         submenu_wl.Append(wl_item)
         self.ID_TO_TOOL_ITEM[new_id] = wl_item
 
-        for name in sorted(const.WINDOW_LEVEL):
+        for name in const.WINDOW_LEVEL:
             if not(name == _('Default') or name == _('Manual')):
                 new_id = wx.NewId()
                 wl_item = wx.MenuItem(submenu_wl, new_id,\
@@ -205,7 +205,7 @@ class SliceMenu(wx.Menu):
     def OnPopup(self, evt):
         id = evt.GetId()
         item = self.ID_TO_TOOL_ITEM[evt.GetId()]
-        key = item.GetLabel()
+        key = item.GetItemLabelText()
         if(key in const.WINDOW_LEVEL.keys()):
             window, level = const.WINDOW_LEVEL[key]
             Publisher.sendMessage('Bright and contrast adjustment image',
@@ -262,8 +262,8 @@ class SliceMenu(wx.Menu):
             if self.cdialog is None:
                 slc = sl.Slice()
                 histogram = slc.histogram
-                init = slc.matrix.min()
-                end = slc.matrix.max()
+                init = int(slc.matrix.min())
+                end = int(slc.matrix.max())
                 nodes = slc.nodes
                 self.cdialog = ClutImagedataDialog(histogram, init, end, nodes)
                 self.cdialog.Show()
