@@ -21,7 +21,7 @@
 
 try:
     import configparser as ConfigParser
-except(ImportError):
+except ImportError:
     import ConfigParser
 
 import locale
@@ -66,22 +66,21 @@ def GetLocaleOS():
 def InstallLanguage(language):
     file_path = os.path.split(__file__)[0]
 
-    if hasattr(sys,"frozen") and (sys.frozen == "windows_exe"\
-                            or sys.frozen == "console_exe"):
+    abs_file_path = os.path.abspath(file_path + os.sep + "..")
 
-        abs_file_path = os.path.abspath(file_path + os.sep + ".." + os.sep + ".." + os.sep + "..")
-        language_dir = os.path.join(abs_file_path, 'locale')
-    else:
-        abs_file_path = os.path.abspath(file_path + os.sep + "..")
-        language_dir = os.path.join(abs_file_path, 'locale')
+    if hasattr(sys, "frozen") and (sys.frozen == "windows_exe" or sys.frozen == "console_exe"):
+        abs_file_path = os.path.abspath(abs_file_path + os.sep + ".." + os.sep + "..")
 
-    # MAC App
+    language_dir = os.path.join(abs_file_path, 'locale')
+
+    # MAC app
     if not os.path.exists(language_dir):
         abs_file_path = os.path.abspath(os.path.join(file_path, '..', '..',  '..', '..'))
         language_dir = os.path.join(abs_file_path, 'locale')
 
-    lang = gettext.translation('invesalius', language_dir,\
-                                   languages=[language], codeset='utf8') 
+    lang = gettext.translation('invesalius', language_dir,
+                               languages=[language], codeset='utf8')
+
     # Using unicode
     try:
         lang.install(unicode=1)
