@@ -17,8 +17,6 @@
 #    detalhes.
 #--------------------------------------------------------------------------
 
-from six import with_metaclass
-
 import datetime
 import glob
 import os
@@ -29,17 +27,18 @@ import tarfile
 import tempfile
 
 import numpy as np
-import wx
-from wx.lib.pubsub import pub as Publisher
 import vtk
+import wx
+
+from invesalius.pubsub import pub as Publisher
 
 import invesalius.constants as const
 import invesalius.data.polydata_utils as pu
-from invesalius.presets import Presets 
-from invesalius.utils import Singleton, debug, touch, decode
 import invesalius.version as version
-
 from invesalius import inv_paths
+from invesalius.data import imagedata_utils
+from invesalius.presets import Presets
+from invesalius.utils import Singleton, debug, decode, touch, TwoWaysDictionary
 
 if sys.platform == 'win32':
     try:
@@ -52,7 +51,7 @@ else:
 
 # Only one project will be initialized per time. Therefore, we use
 # Singleton design pattern for implementing it
-class Project(with_metaclass(Singleton, object)):
+class Project(metaclass=Singleton):
     def __init__(self):
         # Patient/ acquistion information
         self.name = ''
@@ -63,7 +62,7 @@ class Project(with_metaclass(Singleton, object)):
         self.affine = ''
 
         # Masks (vtkImageData)
-        self.mask_dict = {}
+        self.mask_dict = TwoWaysDictionary()
 
         # Surfaces are (vtkPolyData)
         self.surface_dict = {}
@@ -116,16 +115,20 @@ class Project(with_metaclass(Singleton, object)):
         """
         index = len(self.mask_dict)
         self.mask_dict[index] = mask
+        mask.index = index
         return index
 
     def RemoveMask(self, index):
-        new_dict = {}
+        new_dict = TwoWaysDictionary()
+        new_index = 0
         for i in self.mask_dict:
-            if i < index:
-                new_dict[i] = self.mask_dict[i]
-            if i > index:
-                new_dict[i-1] = self.mask_dict[i]
-                new_dict[i-1].index = i-1
+            if i == index:
+                mask = self.mask_dict[i]
+                mask.cleanup()
+            else:
+                new_dict[new_index] = self.mask_dict[i]
+                self.mask_dict[i] = new_index
+                new_index += 1
         self.mask_dict = new_dict
 
     def GetMask(self, index):
@@ -187,7 +190,8 @@ class Project(with_metaclass(Singleton, object)):
 
     def SetRaycastPreset(self, label):
         path = os.path.join(RAYCASTING_PRESETS_DIRECTORY, label + '.plist')
-        preset = plistlib.readPlist(path)
+        with open(path, 'r+b') as f:
+            preset = plistlib.load(f, fmt=plistlib.FMT_XML)
         Publisher.sendMessage('Set raycasting preset', preset)
 
     def GetMeasuresDict(self):
@@ -251,9 +255,9 @@ class Project(with_metaclass(Singleton, object)):
         # Saving the measurements
         measurements = self.GetMeasuresDict()
         measurements_filename = 'measurements.plist'
-        temp_mplist = tempfile.mktemp() 
-        plistlib.writePlist(measurements, 
-                            temp_mplist)
+        temp_mplist = tempfile.mktemp()
+        with open(temp_mplist, 'w+b') as f:
+            plistlib.dump(measurements, f)
         filelist[temp_mplist] = measurements_filename
         project['measurements'] = measurements_filename
 
@@ -262,7 +266,8 @@ class Project(with_metaclass(Singleton, object)):
 
         # Saving the main plist
         temp_plist = tempfile.mktemp()
-        plistlib.writePlist(project, temp_plist)
+        with open(temp_plist, 'w+b') as f:
+            plistlib.dump(project, f)
         filelist[temp_plist] = 'main.plist'
 
         # Compressing and generating the .inv3 file
@@ -277,23 +282,28 @@ class Project(with_metaclass(Singleton, object)):
                 os.remove(f)
 
     def OpenPlistProject(self, filename):
-        import invesalius.data.measures as ms
-        import invesalius.data.mask as msk
-        import invesalius.data.surface as srf
-        
         if not const.VTK_WARNING:
             log_path = os.path.join(inv_paths.USER_LOG_DIR, 'vtkoutput.txt')
             fow = vtk.vtkFileOutputWindow()
             fow.SetFileName(log_path.encode(const.FS_ENCODE))
             ow = vtk.vtkOutputWindow()
             ow.SetInstance(fow)
-            
+
         filelist = Extract(filename, tempfile.mkdtemp())
         dirpath = os.path.abspath(os.path.split(filelist[0])[0])
+        self.load_from_folder(dirpath)
 
+    def load_from_folder(self, dirpath):
+        """
+        Loads invesalius3 project files from dipath.
+        """
+        import invesalius.data.measures as ms
+        import invesalius.data.mask as msk
+        import invesalius.data.surface as srf
         # Opening the main file from invesalius 3 project
         main_plist =  os.path.join(dirpath ,'main.plist')
-        project = plistlib.readPlist(main_plist)
+        with open(main_plist, 'r+b') as f:
+            project = plistlib.load(f, fmt=plistlib.FMT_XML)
 
         format_version = project["format_version"]
         if format_version > const.INVESALIUS_ACTUAL_FORMAT_VERSION:
@@ -308,10 +318,6 @@ class Project(with_metaclass(Singleton, object)):
         self.level = project["window_level"]
         self.threshold_range = project["scalar_range"]
         self.spacing = project["spacing"]
-        if project.get("affine"):
-            self.affine = project["affine"]
-            Publisher.sendMessage('Update affine matrix',
-                                  affine=self.affine, status=True)
 
         self.compress = project.get("compress", True)
 
@@ -321,18 +327,23 @@ class Project(with_metaclass(Singleton, object)):
         self.matrix_shape = project["matrix"]['shape']
         self.matrix_dtype = project["matrix"]['dtype']
 
+        if project.get("affine", ""):
+            self.affine = project["affine"]
+
         # Opening the masks
-        self.mask_dict = {}
-        for index in project["masks"]:
+        self.mask_dict = TwoWaysDictionary()
+        for index in sorted(project.get("masks", []), key=lambda x: int(x)):
             filename = project["masks"][index]
             filepath = os.path.join(dirpath, filename)
             m = msk.Mask()
+            m.spacing = self.spacing
             m.OpenPList(filepath)
+            m.index = len(self.mask_dict)
             self.mask_dict[m.index] = m
 
         # Opening the surfaces
         self.surface_dict = {}
-        for index in project["surfaces"]:
+        for index in sorted(project.get("surfaces", []), key=lambda x: int(x)):
             filename = project["surfaces"][index]
             filepath = os.path.join(dirpath, filename)
             s = srf.Surface(int(index))
@@ -341,15 +352,54 @@ class Project(with_metaclass(Singleton, object)):
 
         # Opening the measurements
         self.measurement_dict = {}
-        measurements = plistlib.readPlist(os.path.join(dirpath,
-                                                       project["measurements"]))
-        for index in measurements:
-            if measurements[index]["type"] in (const.DENSITY_ELLIPSE, const.DENSITY_POLYGON):
-                measure = ms.DensityMeasurement()
-            else:
-                measure = ms.Measurement()
-            measure.Load(measurements[index])
-            self.measurement_dict[int(index)] = measure
+        measures_file = os.path.join(dirpath, project.get("measurements", "measurements.plist"))
+        if os.path.exists(measures_file):
+            with open(measures_file, 'r+b') as f:
+                measurements = plistlib.load(f, fmt=plistlib.FMT_XML)
+            for index in measurements:
+                if measurements[index]["type"] in (const.DENSITY_ELLIPSE, const.DENSITY_POLYGON):
+                    measure = ms.DensityMeasurement()
+                else:
+                    measure = ms.Measurement()
+                measure.Load(measurements[index])
+                self.measurement_dict[int(index)] = measure
+
+    def create_project_file(self, name, spacing, modality, orientation, window_width, window_level, image, affine='', folder=None):
+        if folder is None:
+            folder = tempfile.mkdtemp()
+        if not os.path.exists(folder):
+            os.mkdir(folder)
+        image_file = os.path.join(folder, 'matrix.dat')
+        image_mmap = imagedata_utils.array2memmap(image, image_file)
+        matrix = {
+            'filename': 'matrix.dat',
+            'shape': image.shape,
+            'dtype': str(image.dtype)
+        }
+        project = {
+                   # Format info
+                   "format_version": const.INVESALIUS_ACTUAL_FORMAT_VERSION,
+                   "invesalius_version": const.INVESALIUS_VERSION,
+                   "date": datetime.datetime.now().isoformat(),
+                   "compress": True,
+
+                   # case info
+                   "name": name, # patient's name
+                   "modality": modality, # CT, RMI, ...
+                   "orientation": orientation,
+                   "window_width": window_width,
+                   "window_level": window_level,
+                   "scalar_range": (int(image.min()), int(image.max())),
+                   "spacing": spacing,
+                   "affine": affine,
+
+                   "matrix": matrix,
+                  }
+
+        path = os.path.join(folder, 'main.plist')
+        with open(path, 'w+b') as f:
+            plistlib.dump(project, f)
+
 
     def export_project(self, filename, save_masks=True):
         if filename.lower().endswith('.hdf5') or filename.lower().endswith('.h5'):
