@@ -17,8 +17,6 @@
 #    detalhes.
 #--------------------------------------------------------------------------
 
-from six import with_metaclass
-
 try:
     import configparser as ConfigParser
 except(ImportError):
@@ -33,11 +31,10 @@ import codecs
 import collections
 import json
 
-#import wx.lib.pubsub as ps
-from wx.lib.pubsub import pub as Publisher
+from invesalius.pubsub import pub as Publisher
 import wx
 
-from invesalius.utils import Singleton, debug, decode
+from invesalius.utils import Singleton, debug, decode, deep_merge_dict
 from random import randint
 
 from invesalius import inv_paths
@@ -52,16 +49,18 @@ SESSION_ENCODING = 'utf8'
 
 # Only one session will be initialized per time. Therefore, we use
 # Singleton design pattern for implementing it
-class Session(with_metaclass(Singleton, object)):
+class Session(metaclass=Singleton):
 
     def __init__(self):
         self.project_path = ()
         self.temp_item = False
+        self.mask_3d_preview = False
 
         self._values = collections.defaultdict(dict, {
             'session': {
                 'status': 3,
                 'language': '',
+                'auto_reload_preview': False,
             },
             'project': {
             },
@@ -79,6 +78,7 @@ class Session(with_metaclass(Singleton, object)):
             'surface_interpolation': ('session', 'surface_interpolation'),
             'rendering': ('session', 'rendering'),
             'slice_interpolation': ('session', 'slice_interpolation'),
+            'auto_reload_preview': ('session', 'auto_reload_preview'),
             'recent_projects': ('project', 'recent_projects'),
             'homedir': ('paths', 'homedir'),
             'tempdir': ('paths', 'homedir'),
@@ -88,11 +88,6 @@ class Session(with_metaclass(Singleton, object)):
 
     def CreateItens(self):
         import invesalius.constants as const
-        homedir = inv_paths.USER_DIR
-        tempdir = os.path.join(inv_paths.USER_DIR, u".invesalius", u"temp")
-        if not os.path.isdir(tempdir):
-            os.makedirs(tempdir)
-
         self._values = collections.defaultdict(dict, {
             'session': {
                 'mode': const.MODE_RP,
@@ -103,6 +98,7 @@ class Session(with_metaclass(Singleton, object)):
                 'surface_interpolation': 1,
                 'rendering': 0,
                 'slice_interpolation': 0,
+                'auto_reload_preview': False,
             },
 
             'project': {
@@ -111,7 +107,7 @@ class Session(with_metaclass(Singleton, object)):
 
             'paths': {
                 'homedir': str(inv_paths.USER_DIR),
-                'tempdir': os.path.join(homedir, u".invesalius", u"temp"),
+                'tempdir': str(inv_paths.TEMP_DIR),
                 'last_dicom_folder': '',
             },
         })
@@ -133,7 +129,7 @@ class Session(with_metaclass(Singleton, object)):
         return object.__getattribute__(self, '_values')[session][key]
 
     def __setattr__(self, name, value):
-        if name in ("temp_item", "_map_attrs", "_values", "project_path"):
+        if name in ("temp_item", "_map_attrs", "_values", "project_path", "mask_3d_preview"):
             return object.__setattr__(self, name, value)
         else:
             session, key = self._map_attrs[name]
@@ -276,7 +272,7 @@ class Session(with_metaclass(Singleton, object)):
     def _read_cfg_from_json(self, json_filename):
         with open(json_filename, 'r') as cfg_file:
             cfg_dict = json.load(cfg_file)
-            self._values.update(cfg_dict)
+            self._values = deep_merge_dict(self._values.copy(), cfg_dict)
 
         # Do not reading project status from the config file, since there
         # isn't a recover session tool in InVesalius yet.
@@ -319,6 +315,5 @@ class Session(with_metaclass(Singleton, object)):
             except Exception as e2:
                 debug(e2)
                 return False
-
         self.WriteSessionFile()
         return True
